/******************************************************************************************
PDS_NEXUS_v0_6_4.sql T-SQL script for Microsoft SQL Server 2008 R2 version 10.50.1600

Copyright (c) 2006-2010 Carl Taswell and Global TeleGenetics, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this 
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge, 
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons 
to whom the Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies 
	or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
	INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
	PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
	OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************************/

BEGIN TRANSACTION
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET QUOTED_IDENTIFIER ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET XACT_ABORT ON
COMMIT TRANSACTION
GO

IF EXISTS (select * from tempdb..sysobjects where id = OBJECT_ID('tempdb..#ErrorLog')) 
	DROP TABLE #ErrorLog 
CREATE TABLE #ErrorLog 
( pkid [int] IDENTITY(1,1) NOT NULL, errno [int] NOT NULL, errdescr [varchar](100) NULL )
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[pds_AEntityTypeEnum] (
	[CodeKey] [smallint] NOT NULL,
	[TypeName] [varchar](64) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[TypeDescription] [varchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[TypeEditedByAgent] [bit] NOT NULL,
	[TypeEditedByAuthor] [bit] NOT NULL,
	[TypeEditedByEditor] [bit] NOT NULL,
	[TypeEditedByAdmin] [bit] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_AEntityTypeEnum')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[pds_AInfosetStatusEnum] (
	[CodeKey] [smallint] NOT NULL,
	[StatusName] [varchar](64) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[StatusDescription] [varchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[StatusEditedByAgent] [bit] NOT NULL,
	[StatusEditedByAuthor] [bit] NOT NULL,
	[StatusEditedByEditor] [bit] NOT NULL,
	[StatusEditedByAdmin] [bit] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_AInfosetStatusEnum')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_AUserAccountSystem] (
	[SystemIidKey] [int] IDENTITY (1,1) NOT NULL,
	[SystemName] [nvarchar](64) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[SystemDescription] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[ProviderName] [nvarchar](32) COLLATE Latin1_General_100_CI_AI NULL,
	[DatabaseName] [nvarchar](32) COLLATE Latin1_General_100_CI_AI NULL,
	[SchemaName] [nvarchar](32) COLLATE Latin1_General_100_CI_AI NULL,
	[ObjectQualifierName] [nvarchar](32) COLLATE Latin1_General_100_CI_AI NULL,
	[ApplicationName] [nvarchar](32) COLLATE Latin1_General_100_CI_AI NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_AUserAccountSystem')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NAgent] (
	[AgentIidKey] [int] IDENTITY (1,1) NOT NULL,
	[AgentInfosetGuidRef] [uniqueidentifier] NULL,
	[AgentIsAuthor] [bit] NOT NULL,
	[AgentIsEditor] [bit] NOT NULL,
	[AgentIsAdmin] [bit] NOT NULL,
	[AspnetSystemIidRef] [int] NULL,
	[AspnetUserGuidRef] [uniqueidentifier] NULL,
	[DnnSystemIidRef] [int] NULL,
	[DnnUserIidRef] [int] NULL,
	[OtherSystemIidRef] [int] NULL,
	[OtherUserIidRef] [int] NULL,
	[OtherUserGuidRef] [uniqueidentifier] NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NAgent')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NAgentRegistry] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[AgentIidRef] [int] NOT NULL,
	[RegistryInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NAgentRegistry')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NCrossReference] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[CrossReferenceUri] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[CrossReferenceXml] [xml] NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NCrossReference')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NLocation] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[LocationUrl] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[LocationXml] [xml] NULL,
	[LocationDisplayText] [nvarchar](64) COLLATE Latin1_General_100_CI_AI NULL,
	[LocationDisplayImageUrl] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NLocation')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NRegistryRestrictionAnd] (
	[InternalAndKey] [int] IDENTITY (100001,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[InfosetGuidRef] [uniqueidentifier] NOT NULL,
	[RestrictionIsLabel] [bit] NOT NULL,
	[RestrictionAndIndex] [tinyint] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NRegistryRestrictionAnd')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NRegistryRestrictionOr] (
	[InternalOrKey] [int] IDENTITY (100001,1) NOT NULL,
	[InternalAndRef] [int] NOT NULL,
	[RestrictionOrIndex] [tinyint] NOT NULL,
	[Restriction] [nvarchar](384) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NRegistryRestrictionOr')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[pds_NResource] (
	[EntityHidKey] [hierarchyid] NULL,
	[EntityTypeCode] [smallint] NOT NULL,
	[EntityName] [nvarchar](128) COLLATE Latin1_General_100_CI_AI NULL,
	[EntityNature] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[EntityOwnerInfosetGuidRef] [uniqueidentifier] NULL,
	[EntityContactInfosetGuidRef] [uniqueidentifier] NULL,
	[EntityOtherInfosetGuidRef] [uniqueidentifier] NULL,
	[EntityDescription] [xml] NULL,
	[EntityOtherMetadata] [xml] NULL,
	[ResourceIidKey] [int] IDENTITY (100100,1) NOT NULL,
	[RecordHandle] [char](9) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[RecordIsCachedCopy] [bit] NOT NULL,
	[RecordRegistrantInfosetGuidRef] [uniqueidentifier] NULL,
	[RecordRegistrarInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[RecordRegistryInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[RecordDirectoryInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[RecordCreatedByAgentIidRef] [int] NULL,
	[RecordCreatedOn] [datetime2](7) NOT NULL,
	[RecordUpdatedByAgentIidRef] [int] NULL,
	[RecordUpdatedOn] [datetime2](7) NOT NULL,
	[RecordManagedByAgentIidRef] [int] NULL,
	[RecordVersion] [timestamp] NOT NULL,
	[RecordSignature] [xml] NULL,
	[RecordProvenance] [xml] NULL,
	[RecordDistribution] [xml] NULL,
	[InfosetGuidKey] [uniqueidentifier] ROWGUIDCOL NOT NULL,
	[InfosetIsPrivate] [bit] NOT NULL,
	[InfosetIsVerbose] [bit] NOT NULL,
	[InfosetEntailment] [xml] NULL,
	[InfosetPortalStatusCode] [smallint] NOT NULL,
	[InfosetPortalStatusTestedOn] [datetime2](7) NULL,
	[InfosetDoorsStatusCode] [smallint] NOT NULL,
	[InfosetDoorsStatusTestedOn] [datetime2](7) NULL
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NResource')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NSecondaryDirectory] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[DirectoryInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NSecondaryDirectory')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NSecondaryRegistry] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[RegistryInfosetGuidRef] [uniqueidentifier] NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NSecondaryRegistry')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NSupportingLabel] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[SupportingLabel] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[LabelIsRestricted] [bit] NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NSupportingLabel')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NSupportingTag] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[SupportingTag] [nvarchar](128) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[TagIsRestricted] [bit] NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NSupportingTag')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[pds_NTagAndLabel] (
	[InternalKey] [int] IDENTITY (1,1) NOT NULL,
	[ResourceIidRef] [int] NOT NULL,
	[Priority] [tinyint] NOT NULL,
	[LabelIsCanonical] [bit] NOT NULL,
	[LabelIsPrivate] [bit] NOT NULL,
	[LabelIsResolvable] [bit] NOT NULL,
	[LabelIsTagIndependent] [bit] NOT NULL,
	[TagToken] [nvarchar](128) COLLATE Latin1_General_100_CI_AI NULL,
	[LabelUri] [nvarchar](256) COLLATE Latin1_General_100_CI_AI NULL,
	[EntityLabel] [nvarchar](384) COLLATE Latin1_General_100_CI_AI NOT NULL,
	[Version] [timestamp] NOT NULL
) ON [PRIMARY]
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add table dbo.pds_NTagAndLabel')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO





CREATE VIEW [dbo].[pds_NAgentRegistryView]
AS
SELECT InternalKey, AgentIidRef, RegistryInfosetGuidRef
FROM  pds_NAgentRegistry






GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NAgentRegistryView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO




CREATE VIEW [dbo].[pds_NAgentView]
AS
SELECT AgentIidKey, AgentInfosetGuidRef, AspnetUserGuidRef, 
   AgentIsAuthor, AgentIsEditor, AgentIsAdmin
FROM  pds_NAgent





GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NAgentView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO







CREATE VIEW [dbo].[pds_NCrossReferenceView]
  
AS

SELECT InternalKey, ResourceIidRef, Priority,
CrossReferenceUri, CrossReferenceXml
FROM pds_NCrossReference 







GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NCrossReferenceView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO











CREATE VIEW [dbo].[pds_NEntityAliasLabelView]
AS
SELECT entRRtl.InternalKey, entRRtl.ResourceIidRef, entRRtl.Priority, 
entRRtl.LabelIsCanonical, entRRtl.LabelIsPrivate, entRRtl.LabelIsResolvable, entRRtl.LabelIsTagIndependent,
entRRtl.TagToken AS EntityAliasTag, 
entRRtl.LabelUri As EntityAliasUri,
entRRtl.EntityLabel AS EntityAliasLabel
FROM pds_NResource AS RR INNER 
JOIN pds_NTagAndLabel AS entRRtl 
ON (RR.ResourceIidKey = entRRtl.ResourceIidRef) AND (entRRtl.LabelIsCanonical = 0)






GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NEntityAliasLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO










CREATE VIEW [dbo].[pds_NEntityCanonicalLabelView]
AS
SELECT  RR.EntityName, 
entRRtl.EntityLabel AS EntityCanonicalLabel,
entRRtl.LabelIsPrivate AS EntityLabelIsPrivate,
entRRtl.LabelIsResolvable AS EntityLabelIsResolvable,
entRRtl.TagToken AS EntityPrincipalTag, 
entRRtl.InternalKey AS EntityTagLabelInternalKey,
RR.ResourceIidKey, RR.EntityHidKey, RR.RecordHandle, RR.InfosetGuidKey
FROM pds_NResource AS RR INNER 
JOIN pds_NTagAndLabel AS entRRtl 
ON (RR.ResourceIidKey = entRRtl.ResourceIidRef) AND (entRRtl.LabelIsCanonical = 1)











GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NEntityCanonicalLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO














CREATE VIEW [dbo].[pds_NEntityLabelView]
AS
SELECT entRRtl.InternalKey, entRRtl.ResourceIidRef, 
entRRtl.Priority, entRRtl.LabelIsPrivate, entRRtl.LabelIsResolvable,
entRRtl.TagToken AS EntityTag, 
entRRtl.LabelUri AS EntityUri,
entRRtl.EntityLabel,
RR.RecordHandle, RR.InfosetGuidKey
FROM pds_NResource AS RR INNER 
JOIN pds_NTagAndLabel AS entRRtl 
ON (RR.ResourceIidKey = entRRtl.ResourceIidRef) 





GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NEntityLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO



CREATE VIEW [dbo].[pds_NLocationView]
AS
SELECT InternalKey, ResourceIidRef, Priority,
LocationUrl, LocationXml, LocationDisplayText, LocationDisplayImageUrl
FROM  pds_NLocation




GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NLocationView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO














CREATE VIEW [dbo].[pds_NRecordRegistryLabelView]
AS
SELECT RR.ResourceIidKey, 
RR.RecordRegistryInfosetGuidRef AS RecordRegistryGuidRef,
rgyRR.EntityName AS RecordRegistryName, 
rgyRRtl.TagToken AS RecordRegistryTag, 
rgyRRtl.EntityLabel AS RecordRegistryLabel
FROM pds_NResource AS RR INNER 
JOIN pds_NResource AS rgyRR ON (RR.RecordRegistryInfosetGuidRef = rgyRR.InfosetGuidKey) INNER
JOIN pds_NTagAndLabel As rgyRRtl ON (rgyRR.ResourceIidKey = rgyRRtl.ResourceIidRef) AND (rgyRRtl.LabelIsCanonical = 1)





GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NRecordRegistryLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO







CREATE VIEW [dbo].[pds_NRegistryEditorView]
AS
SELECT E.InternalKey, E.AgentIidRef, R.RecordRegistryInfosetGuidRef, R.ResourceIidKey
FROM  pds_NResource AS R INNER
JOIN pds_NAgentRegistry AS E ON R.RecordRegistryInfosetGuidRef = E.RegistryInfosetGuidRef INNER
JOIN pds_NAgent AS A ON E.AgentIidRef = A.AgentIidKey  AND A.AgentIsEditor = 1 








GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NRegistryEditorView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


CREATE VIEW [dbo].[pds_NRegistryRestrictionView]
AS
SELECT 
RA.InternalAndKey, RA.ResourceIidRef, RA.InfosetGuidRef, RA.RestrictionIsLabel, RA.RestrictionAndIndex, 
RO.InternalOrKey,  RO.RestrictionOrIndex, RO.Restriction
FROM  pds_NRegistryRestrictionAnd AS RA INNER 
JOIN  pds_NRegistryRestrictionOr AS RO
ON RA.InternalAndKey = RO.InternalAndRef



GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NRegistryRestrictionView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO











CREATE VIEW [dbo].[pds_NSupportingLabelView]
  
AS

SELECT InternalKey, ResourceIidRef, Priority,
	SupportingLabel, LabelIsRestricted
FROM pds_NSupportingLabel








GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NSupportingLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO










CREATE VIEW [dbo].[pds_NSupportingTagView]
  
AS

SELECT InternalKey, ResourceIidRef, Priority,
	SupportingTag, TagIsRestricted
FROM pds_NSupportingTag







GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NSupportingTagView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO






CREATE VIEW [dbo].[pds_NTagAndLabelView]
AS
SELECT InternalKey, ResourceIidRef, Priority, 
LabelIsCanonical, LabelIsPrivate, LabelIsResolvable, LabelIsTagIndependent, 
TagToken, LabelUri, EntityLabel
FROM pds_NTagAndLabel







GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NTagAndLabelView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AEntityTypeEnum] ADD 
	CONSTRAINT [DF_AEntityTypeEnum_TypeEditedByAdmin] DEFAULT ((1)) FOR [TypeEditedByAdmin]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AEntityTypeEnum] ADD 
	CONSTRAINT [DF_AEntityTypeEnum_TypeEditedByAgent] DEFAULT ((0)) FOR [TypeEditedByAgent]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AEntityTypeEnum] ADD 
	CONSTRAINT [DF_AEntityTypeEnum_TypeEditedByAuthor] DEFAULT ((0)) FOR [TypeEditedByAuthor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AEntityTypeEnum] ADD 
	CONSTRAINT [DF_AEntityTypeEnum_TypeEditedByEditor] DEFAULT ((0)) FOR [TypeEditedByEditor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AInfosetStatusEnum] ADD 
	CONSTRAINT [DF_AInfosetStatusEnum_StatusEditedByAdmin] DEFAULT ((1)) FOR [StatusEditedByAdmin]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AInfosetStatusEnum] ADD 
	CONSTRAINT [DF_AInfosetStatusEnum_StatusEditedByAgent] DEFAULT ((0)) FOR [StatusEditedByAgent]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AInfosetStatusEnum] ADD 
	CONSTRAINT [DF_AInfosetStatusEnum_StatusEditedByAuthor] DEFAULT ((0)) FOR [StatusEditedByAuthor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AInfosetStatusEnum] ADD 
	CONSTRAINT [DF_AInfosetStatusEnum_StatusEditedByEditor] DEFAULT ((0)) FOR [StatusEditedByEditor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] ADD 
	CONSTRAINT [DF_NAgentIsAdmin] DEFAULT ((0)) FOR [AgentIsAdmin]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] ADD 
	CONSTRAINT [DF_NAgentIsAuthor] DEFAULT ((1)) FOR [AgentIsAuthor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] ADD 
	CONSTRAINT [DF_NAgentIsEditor] DEFAULT ((0)) FOR [AgentIsEditor]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NCrossReference] ADD 
	CONSTRAINT [DF_NCrossReference_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NLocation] ADD 
	CONSTRAINT [DF_NLocation_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceEntityTypeCode] DEFAULT ((0)) FOR [EntityTypeCode]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceInfosetDoorsStatusCode] DEFAULT ((0)) FOR [InfosetDoorsStatusCode]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceInfosetGuidKey] DEFAULT (newid()) FOR [InfosetGuidKey]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceInfosetIsPrivate] DEFAULT ((0)) FOR [InfosetIsPrivate]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceInfosetIsVerbose] DEFAULT ((0)) FOR [InfosetIsVerbose]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceRecordCreatedOn] DEFAULT (getutcdate()) FOR [RecordCreatedOn]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceRecordIsCachedCopy] DEFAULT ((0)) FOR [RecordIsCachedCopy]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourceRecordUpdatedOn] DEFAULT (getutcdate()) FOR [RecordUpdatedOn]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [DF_NResourcesInfosetPortalStatusCode] DEFAULT ((0)) FOR [InfosetPortalStatusCode]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryDirectory] ADD 
	CONSTRAINT [DF_NSecondaryDirectory_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryRegistry] ADD 
	CONSTRAINT [DF_NSecondaryRegistry_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingLabel] ADD 
	CONSTRAINT [DF_NSupportingLabel_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingTag] ADD 
	CONSTRAINT [DF_NSupportingTag_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_NTagAndLabel_EntityLabel] DEFAULT (N'newguid()') FOR [EntityLabel]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_NTagAndLabel_LabelIsCanonical] DEFAULT ((0)) FOR [LabelIsCanonical]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_NTagAndLabel_LabelIsPrivate] DEFAULT ((0)) FOR [LabelIsPrivate]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_NTagAndLabel_LabelIsTagIndependent] DEFAULT ((0)) FOR [LabelIsTagIndependent]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_NTagAndLabel_Priority] DEFAULT ((0)) FOR [Priority]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionAnd] ADD 
	CONSTRAINT [DF_pds_NRegistryRestrictionAnd_RestrictionIsLabel] DEFAULT ((0)) FOR [RestrictionIsLabel]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingLabel] ADD 
	CONSTRAINT [DF_pds_NSupportingLabel_LabelIsRestricted] DEFAULT ((0)) FOR [LabelIsRestricted]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingTag] ADD 
	CONSTRAINT [DF_pds_NSupportingTag_TagIsRestricted] DEFAULT ((0)) FOR [TagIsRestricted]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [DF_pds_NTagAndLabel_LabelIsResolvable] DEFAULT ((0)) FOR [LabelIsResolvable]
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO











CREATE VIEW [dbo].[pds_NResourceView]
  
AS
SELECT RR.EntityHidKey, 
	RR.EntityTypeCode, TC.TypeName AS EntityTypeName,
	RR.EntityName, RR.EntityNature, 
	ettRR.EntityTagLabelInternalKey,
	ettRR.EntityPrincipalTag, ettRR.EntityCanonicalLabel, 
	RR.EntityDescription, RR.EntityOtherMetadata, 
	RR.EntityOwnerInfosetGuidRef As EntityOwnerGuidRef,				onrRR.EntityName AS EntityOwnerName,	  onrRR.EntityPrincipalTag AS EntityOwnerTag,      onrRR.EntityCanonicalLabel AS EntityOwnerLabel,   
	RR.EntityContactInfosetGuidRef AS EntityContactGuidRef,			ctcRR.EntityName AS EntityContactName,    ctcRR.EntityPrincipalTag AS EntityContactTag,    ctcRR.EntityCanonicalLabel AS EntityContactLabel,   
	RR.EntityOtherInfosetGuidRef AS EntityOtherGuidRef,				othRR.EntityName AS EntityOtherName,	  othRR.EntityPrincipalTag AS EntityOtherTag,      othRR.EntityCanonicalLabel AS EntityOtherLabel,   
	RR.ResourceIidKey, 
	RR.RecordHandle, 
	RR.RecordIsCachedCopy, 
	RR.RecordRegistrantInfosetGuidRef AS RecordRegistrantGuidRef,	rgtRR.EntityName AS RecordRegistrantName, rgtRR.EntityPrincipalTag AS RecordRegistrantTag, rgtRR.EntityCanonicalLabel AS RecordRegistrantLabel,
	RR.RecordRegistrarInfosetGuidRef AS RecordRegistrarGuidRef,		rgrRR.EntityName AS RecordRegistrarName,  rgrRR.EntityPrincipalTag AS RecordRegistrarTag,  rgrRR.EntityCanonicalLabel AS RecordRegistrarLabel,
	RR.RecordRegistryInfosetGuidRef AS RecordRegistryGuidRef,		rgsRR.EntityName AS RecordRegistryName,   rgsRR.EntityPrincipalTag AS RecordRegistryTag,   rgsRR.EntityCanonicalLabel AS RecordRegistryLabel,   
	RR.RecordDirectoryInfosetGuidRef AS RecordDirectoryGuidRef,		drcRR.EntityName AS RecordDirectoryName,  drcRR.EntityPrincipalTag AS RecordDirectoryTag,  drcRR.EntityCanonicalLabel AS RecordDirectoryLabel,
	RR.RecordCreatedByAgentIidRef, RR.RecordCreatedOn, RR.RecordUpdatedByAgentIidRef, RR.RecordUpdatedOn, RR.RecordManagedByAgentIidRef, 
	RR.RecordSignature, RR.RecordProvenance, RR.RecordDistribution, 
	RR.InfosetGuidKey, 
	RR.InfosetIsPrivate, RR.InfosetIsVerbose,
	RR.InfosetEntailment,
	RR.InfosetPortalStatusTestedOn, RR.InfosetPortalStatusCode, 
	pSC.StatusName AS InfosetPortalStatusName,
	RR.InfosetDoorsStatusTestedOn, RR.InfosetDoorsStatusCode,
	dSC.StatusName AS InfosetDoorsStatusName
FROM pds_NResource AS RR INNER 
JOIN pds_AEntityTypeEnum AS TC ON (RR.EntityTypeCode = TC.CodeKey) INNER 
JOIN pds_AInfosetStatusEnum AS pSC ON (RR.InfosetPortalStatusCode = pSC.CodeKey) INNER 
JOIN pds_AInfosetStatusEnum AS dSC ON (RR.InfosetDoorsStatusCode = dSC.CodeKey) INNER 
JOIN pds_NEntityCanonicalLabelView AS ettRR ON RR.InfosetGuidKey = ettRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS rgrRR ON RR.RecordRegistrarInfosetGuidRef  = rgrRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS rgsRR ON RR.RecordRegistryInfosetGuidRef   = rgsRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS drcRR ON RR.RecordDirectoryInfosetGuidRef  = drcRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS rgtRR ON RR.RecordRegistrantInfosetGuidRef = rgtRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS ctcRR ON RR.EntityContactInfosetGuidRef    = ctcRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS onrRR ON RR.EntityOwnerInfosetGuidRef      = onrRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS othRR ON RR.EntityOtherInfosetGuidRef      = othRR.InfosetGuidKey










GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NResourceView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO












CREATE VIEW [dbo].[pds_NResourceViewNoHid]
  
AS
SELECT 
	RR.EntityTypeCode, TC.TypeName AS EntityTypeName,
	RR.EntityName, RR.EntityNature, 
	ettRR.EntityTagLabelInternalKey,
	ettRR.EntityPrincipalTag, ettRR.EntityCanonicalLabel, 
	RR.EntityDescription, RR.EntityOtherMetadata, 
	RR.EntityOwnerInfosetGuidRef As EntityOwnerGuidRef,				onrRR.EntityName AS EntityOwnerName,	  onrRR.EntityPrincipalTag AS EntityOwnerTag,      onrRR.EntityCanonicalLabel AS EntityOwnerLabel,   
	RR.EntityContactInfosetGuidRef AS EntityContactGuidRef,			ctcRR.EntityName AS EntityContactName,    ctcRR.EntityPrincipalTag AS EntityContactTag,    ctcRR.EntityCanonicalLabel AS EntityContactLabel,   
	RR.EntityOtherInfosetGuidRef AS EntityOtherGuidRef,				othRR.EntityName AS EntityOtherName,	  othRR.EntityPrincipalTag AS EntityOtherTag,      othRR.EntityCanonicalLabel AS EntityOtherLabel,   
	RR.ResourceIidKey, 
	RR.RecordHandle, 
	RR.RecordIsCachedCopy, 
	RR.RecordRegistrantInfosetGuidRef AS RecordRegistrantGuidRef,	rgtRR.EntityName AS RecordRegistrantName, rgtRR.EntityPrincipalTag AS RecordRegistrantTag, rgtRR.EntityCanonicalLabel AS RecordRegistrantLabel,
	RR.RecordRegistrarInfosetGuidRef AS RecordRegistrarGuidRef,		rgrRR.EntityName AS RecordRegistrarName,  rgrRR.EntityPrincipalTag AS RecordRegistrarTag,  rgrRR.EntityCanonicalLabel AS RecordRegistrarLabel,
	RR.RecordRegistryInfosetGuidRef AS RecordRegistryGuidRef,		rgsRR.EntityName AS RecordRegistryName,   rgsRR.EntityPrincipalTag AS RecordRegistryTag,   rgsRR.EntityCanonicalLabel AS RecordRegistryLabel,   
	RR.RecordDirectoryInfosetGuidRef AS RecordDirectoryGuidRef,		drcRR.EntityName AS RecordDirectoryName,  drcRR.EntityPrincipalTag AS RecordDirectoryTag,  drcRR.EntityCanonicalLabel AS RecordDirectoryLabel,
	RR.RecordCreatedByAgentIidRef, RR.RecordCreatedOn, RR.RecordUpdatedByAgentIidRef, RR.RecordUpdatedOn, RR.RecordManagedByAgentIidRef, 
	RR.RecordSignature, RR.RecordProvenance, RR.RecordDistribution, 
	RR.InfosetGuidKey, 
	RR.InfosetIsPrivate, RR.InfosetIsVerbose,
	RR.InfosetEntailment,
	RR.InfosetPortalStatusTestedOn, RR.InfosetPortalStatusCode, 
	pSC.StatusName AS InfosetPortalStatusName,
	RR.InfosetDoorsStatusTestedOn, RR.InfosetDoorsStatusCode,
	dSC.StatusName AS InfosetDoorsStatusName
FROM pds_NResource AS RR INNER 
JOIN pds_AEntityTypeEnum AS TC ON (RR.EntityTypeCode = TC.CodeKey) INNER 
JOIN pds_AInfosetStatusEnum AS pSC ON (RR.InfosetPortalStatusCode = pSC.CodeKey) INNER 
JOIN pds_AInfosetStatusEnum AS dSC ON (RR.InfosetDoorsStatusCode = dSC.CodeKey) INNER 
JOIN pds_NEntityCanonicalLabelView AS ettRR ON RR.InfosetGuidKey = ettRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS rgrRR ON RR.RecordRegistrarInfosetGuidRef  = rgrRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS rgsRR ON RR.RecordRegistryInfosetGuidRef   = rgsRR.InfosetGuidKey INNER
JOIN pds_NEntityCanonicalLabelView AS drcRR ON RR.RecordDirectoryInfosetGuidRef  = drcRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS rgtRR ON RR.RecordRegistrantInfosetGuidRef = rgtRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS ctcRR ON RR.EntityContactInfosetGuidRef    = ctcRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS onrRR ON RR.EntityOwnerInfosetGuidRef      = onrRR.InfosetGuidKey LEFT OUTER
JOIN pds_NEntityCanonicalLabelView AS othRR ON RR.EntityOtherInfosetGuidRef      = othRR.InfosetGuidKey











GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NResourceViewNoHid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO











CREATE VIEW [dbo].[pds_NSecondaryDirectoryView]
  
AS
SELECT NSD.InternalKey, NSD.ResourceIidRef, NSD.Priority,
	NSD.DirectoryInfosetGuidRef As SecondaryDirectoryGuidRef, 
	nsdNR.EntityName AS SecondaryDirectoryName, 
    nsdNR.EntityCanonicalLabel AS SecondaryDirectoryLabel
FROM pds_NSecondaryDirectory AS NSD INNER 
JOIN pds_NEntityCanonicalLabelView AS nsdNR 
ON NSD.DirectoryInfosetGuidRef = nsdNR.InfosetGuidKey





GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NSecondaryDirectoryView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO










CREATE VIEW [dbo].[pds_NSecondaryRegistryView]
  
AS
SELECT NSR.InternalKey, NSR.ResourceIidRef, NSR.Priority,
	NSR.RegistryInfosetGuidRef As SecondaryRegistryGuidRef, 
	nsrNR.EntityName AS SecondaryRegistryName, 
    nsrNR.EntityCanonicalLabel AS SecondaryRegistryLabel
FROM pds_NSecondaryRegistry AS NSR INNER 
JOIN pds_NEntityCanonicalLabelView AS nsrNR 
ON NSR.RegistryInfosetGuidRef = nsrNR.InfosetGuidKey











GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add view dbo.pds_NSecondaryRegistryView')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_ARandomHexStrVarChar]
(@NumChars tinyint = 9,
 @HexStr varchar(12) output)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@NumChars < 1) OR (@NumChars > 12) RETURN -1;
	
	-- TODO: recode to delete four hyphens in newid and allow up to (@NumChars > 32)

	SET @HexStr = CHAR(ROUND(25*RAND()+65,0)) + RIGHT(CAST(NEWID() AS char(36)), (@NumChars-1));
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

-- test with T-SQL code:
--		declare @nc tinyint, @res varchar(36), @ret int;
--		set @nc = 12;
--		exec @ret = pds_ARandomHexStrVarChar @nc, @res output;
--		select @ret, @res;

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_ARandomHexStrVarChar')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_ADdlistEntityTypeCodes]
(@ForAuthor bit = 0,
 @ForEditor bit = 0,
 @ForAdmin bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@ForAuthor = 1)
		SELECT * FROM pds_AEntityTypeEnum
			WHERE (TypeEditedByAuthor = 1) 
			ORDER BY CodeKey;
			
	ELSE IF (@ForEditor = 1)
		SELECT * FROM pds_AEntityTypeEnum
			WHERE (TypeEditedByEditor = 1) 
			ORDER BY CodeKey;

	ELSE IF (@ForAdmin = 1)	
		SELECT * FROM pds_AEntityTypeEnum
			WHERE (TypeEditedByAdmin = 1) 
			ORDER BY CodeKey;
	
	ELSE	
		SELECT * FROM pds_AEntityTypeEnum
			ORDER BY CodeKey;
		
	RETURN ISNULL(@@ROWCOUNT,0);
	
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_ADdlistEntityTypeCodes')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_ADdlistInfosetStatusCodes]
(@ForAuthor bit = 0,
 @ForEditor bit = 0,
 @ForAdmin bit = 0)
AS
BEGIN

	SET NOCOUNT ON;

	IF (@ForAuthor = 1)
		SELECT * FROM pds_AInfosetStatusEnum
			WHERE (StatusEditedByAuthor = 1) 
			ORDER BY StatusName;
		
	ELSE IF (@ForEditor = 1)
		SELECT * FROM pds_AInfosetStatusEnum
			WHERE (StatusEditedByEditor = 1) 
			ORDER BY StatusName;

	ELSE IF (@ForAdmin = 1)	
		SELECT * FROM pds_AInfosetStatusEnum
			WHERE (StatusEditedByAdmin = 1) 
			ORDER BY StatusName;

	ELSE	
		SELECT * FROM pds_AInfosetStatusEnum
			ORDER BY StatusName;

	RETURN ISNULL(@@ROWCOUNT,0);
	
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_ADdlistInfosetStatusCodes')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NCrossReferenceDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NCrossReference
		WHERE InternalKey = @InternalKey;
 
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NCrossReferenceDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NCrossReferenceInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@CrossReferenceUri nvarchar(256),
@CrossReferenceXml xml)
AS
BEGIN

	SET NOCOUNT ON;

	INSERT INTO pds_NCrossReference
		(ResourceIidRef, Priority, CrossReferenceUri, CrossReferenceXml) 
		VALUES 
		(@ResourceIidRef, @Priority, @CrossReferenceUri, @CrossReferenceXml);
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NCrossReferenceInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NCrossReferenceUpdate](
@Priority tinyint,
@CrossReferenceUri nvarchar(256),
@CrossReferenceXml xml,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
  
    UPDATE pds_NCrossReference
	    SET Priority = @Priority,
	    CrossReferenceUri = @CrossReferenceUri,
	    CrossReferenceXml = @CrossReferenceXml
		WHERE (InternalKey = @InternalKey);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NCrossReferenceUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NDdlistDirectories]
(@ForAuthor bit = 0,
 @ForEditor bit = 0,
 @ForAdmin bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
	
	/* access privilege should go from lowest to highest */
	
	IF (@ForAuthor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 21)
			ORDER BY EntityName;
			
	ELSE IF (@ForEditor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 21 OR EntityTypeCode = 22)
			ORDER BY EntityName;
			
	ELSE IF (@ForAdmin = 1)	
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode >= 20 AND EntityTypeCode <= 22)
			OR (EntityTypeCode = 1)
			ORDER BY EntityName;

	RETURN ISNULL(@@ROWCOUNT,0);

END
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NDdlistDirectories')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NDdlistRegistrars]
(@ForAuthor bit = 0,
 @ForEditor bit = 0,
 @ForAdmin bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
	
	/* access privilege should go from lowest to highest */
	
	IF (@ForAuthor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 31)
			ORDER BY EntityName;
			
	ELSE IF (@ForEditor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 31 OR EntityTypeCode = 32)
			ORDER BY EntityName;
			
	ELSE IF (@ForAdmin = 1)	
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode >= 30 AND EntityTypeCode <= 32)
			OR (EntityTypeCode = 1)
			ORDER BY EntityName;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NDdlistRegistrars')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NDdlistRegistries]
(@ForAuthor bit = 0,
 @ForEditor bit = 0,
 @ForAdmin bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
	
	/* access privilege should go from lowest to highest */
	
	/* TODO: rewrite this storproc to fix the hack on MeSh2010
	   and to check access to each registry as author/editor 
	    so that personalized dropdownlists can be created */
	
	IF (@ForAuthor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 11) And (ResourceIidKey <> 100116)
			ORDER BY EntityName;
			
	ELSE IF (@ForEditor = 1)
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode = 11 OR EntityTypeCode = 12) And (ResourceIidKey <> 100116)
			ORDER BY EntityName;
			
	ELSE IF (@ForAdmin = 1)	
		SELECT EntityName, InfosetGuidKey FROM pds_NResource
			WHERE (EntityTypeCode >= 10 AND EntityTypeCode <= 12) And (ResourceIidKey <> 100116)
			OR (EntityTypeCode = 1)
			ORDER BY EntityName;

	RETURN ISNULL(@@ROWCOUNT,0);

END
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NDdlistRegistries')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NDdlistSecondaryDirectories]
	-- Currently configured to assume that primary can also serve as secondary
AS
BEGIN

	SET NOCOUNT ON;

	SELECT EntityName, InfosetGuidKey
		FROM pds_NResource
		WHERE (EntityTypeCode = 21) OR (EntityTypeCode = 22)
		ORDER BY EntityName;
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NDdlistSecondaryDirectories')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NDdlistSecondaryRegistries]
	-- Currently configured to assume that primary can also serve as secondary
AS
BEGIN

	SET NOCOUNT ON;

	SELECT EntityName, InfosetGuidKey
		FROM pds_NResource
		WHERE (EntityTypeCode = 11) OR (EntityTypeCode = 12)
		ORDER BY EntityName;
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NDdlistSecondaryRegistries')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NGtgpdsAgentIdsCreateFromAspnetUserGuid]
(@AspnetSystemIid int,
 @AspnetUserGuid uniqueidentifier,
 @PdsAgentIid int output,
 @PdsAgentInfosetGuid uniqueidentifier output)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@AspnetSystemIid IS NULL) RETURN -11;
	IF (@AspnetUserGuid IS NULL) RETURN -12;		
	IF (EXISTS(SELECT AgentIidKey FROM pds_NAgent
		WHERE (AspnetSystemIidRef = @AspnetSystemIid) AND (AspnetUserGuidRef = @AspnetUserGuid) )) RETURN -13;
 
    INSERT INTO pds_NAgent (AspnetSystemIidRef, AspnetUserGuidRef) VALUES (@AspnetSystemIid, @AspnetUserGuid);

	SELECT @PdsAgentIid = AgentIidKey, @PdsAgentInfosetGuid = AgentInfosetGuidRef
		FROM pds_NAgent WHERE (AspnetSystemIidRef = @AspnetSystemIid) AND (AspnetUserGuidRef = @AspnetUserGuid);
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NGtgpdsAgentIdsCreateFromAspnetUserGuid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO



-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NGtgpdsAgentIdsCreateFromDnnUserIid]
(	@DnnSystemIid int,
	@DnnUserIid int,
	@AspnetSystemIid int,
	@AspnetUserGuid uniqueidentifier,
	@PdsAgentIid int output,
	@PdsAgentInfosetGuid uniqueidentifier output)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@DnnSystemIid IS NULL) RETURN -1;
	IF (@DnnUserIid IS NULL) RETURN -2;
	IF (EXISTS(SELECT AgentIidKey FROM pds_NAgent 
		WHERE (DnnSystemIidRef = @DnnSystemIid) AND (DnnUserIidRef = @DnnUserIid) )) RETURN -3;
	
	IF (@AspnetSystemIid IS NULL) RETURN -11;
	IF (@AspnetUserGuid IS NULL) RETURN -12;		
	IF (EXISTS(SELECT AgentIidKey 
		FROM pds_NAgent 
		WHERE (AspnetSystemIidRef = @AspnetSystemIid) AND (AspnetUserGuidRef = @AspnetUserGuid) )) RETURN -13;
 
    INSERT INTO pds_NAgent (DnnSystemIidRef, DnnUserIidRef, AspnetSystemIidRef, AspnetUserGuidRef) 
		VALUES (@DnnSystemIid, @DnnUserIid, @AspnetSystemIid, @AspnetUserGuid);

	SELECT @PdsAgentIid = AgentIidKey, @PdsAgentInfosetGuid = AgentInfosetGuidRef
		FROM pds_NAgent 
		WHERE (AspnetSystemIidRef = @AspnetSystemIid) AND (AspnetUserGuidRef = @AspnetUserGuid);
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NGtgpdsAgentIdsCreateFromDnnUserIid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NGtgpdsAgentIdsGetFromAspnetUserGuid]
(@AspnetSystemIid int,
 @AspnetUserGuid uniqueidentifier,
 @PdsAgentIid int output,
 @PdsAgentInfosetGuid uniqueidentifier output,
 @PdsAgentIsAuthor bit output,
 @PdsAgentIsEditor bit output,
 @PdsAgentIsAdmin bit output)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@AspnetSystemIid IS NULL) RETURN -1;
	IF (@AspnetUserGuid IS NULL) RETURN -2;

	SELECT @PdsAgentIid = AgentIidKey, @PdsAgentInfosetGuid = AgentInfosetGuidRef, 
		@PdsAgentIsAuthor = AgentIsAuthor, @PdsAgentIsEditor = AgentIsEditor, @PdsAgentIsAdmin = AgentIsAdmin
		FROM pds_NAgent WHERE (AspnetSystemIidRef = @AspnetSystemIid) AND (AspnetUserGuidRef = @AspnetUserGuid);
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NGtgpdsAgentIdsGetFromAspnetUserGuid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO



-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NGtgpdsAgentIdsGetFromDnnUserIid]
(@DnnSystemIid int,
 @DnnUserIid int,
 @PdsAgentIid int output,
 @PdsAgentInfosetGuid uniqueidentifier output,
 @PdsAgentIsAuthor bit output,
 @PdsAgentIsEditor bit output,
 @PdsAgentIsAdmin bit output)
AS
BEGIN

	SET NOCOUNT ON;
	
	IF (@DnnSystemIid IS NULL) RETURN -1;
	IF (@DnnUserIid IS NULL) RETURN -2;

	SELECT @PdsAgentIid = AgentIidKey, @PdsAgentInfosetGuid = AgentInfosetGuidRef, 
		@PdsAgentIsAuthor = AgentIsAuthor, @PdsAgentIsEditor = AgentIsEditor, @PdsAgentIsAdmin = AgentIsAdmin
		FROM pds_NAgent WHERE (DnnSystemIidRef = @DnnSystemIid) AND (DnnUserIidRef = @DnnUserIid);
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NGtgpdsAgentIdsGetFromDnnUserIid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NLocationDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NLocation
		WHERE (InternalKey = @InternalKey);
 
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NLocationDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NLocationInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@LocationUrl nvarchar(256) = null,
@LocationXml xml = null,
@LocationDisplayText nvarchar(64) = null,
@LocationDisplayImageUrl nvarchar(256) = null)
AS
BEGIN

	SET NOCOUNT ON;

    INSERT INTO dbo.pds_NLocation
		(ResourceIidRef, Priority, LocationUrl, LocationXml,
		 LocationDisplayText, LocationDisplayImageUrl) 
		VALUES 
		(@ResourceIidRef, @Priority, @LocationUrl, @LocationXml,
		 @LocationDisplayText, @LocationDisplayImageUrl); 

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NLocationInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NLocationUpdate](
@Priority tinyint,
@LocationUrl nvarchar(256) = null,
@LocationXml xml = null,
@LocationDisplayText nvarchar(64) = null,
@LocationDisplayImageUrl nvarchar(256) = null,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    UPDATE pds_NLocation
		SET Priority = @Priority,
		LocationUrl = @LocationUrl,
		LocationXml = @LocationXml,
		LocationDisplayText = @LocationDisplayText,
		LocationDisplayImageUrl = @LocationDisplayImageUrl
		WHERE (InternalKey = @InternalKey);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NLocationUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NRegistryUpdateDependentEntityLabels]
(@SetRegistryGuid uniqueidentifier = '00000000-0000-0000-0000-000000000000')
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	DECLARE @InternalKey int, @ResourceIid int, @TagToken nvarchar(128), 
		@LabelUri nvarchar(256), @EntityLabel nvarchar(384), @RegistryLabel nvarchar(384);

	DECLARE labels CURSOR FOR
		SELECT InternalKey, ResourceIidRef, TagToken, LabelUri FROM pds_NTagAndLabel;

	OPEN labels
	FETCH labels INTO @InternalKey, @ResourceIid, @TagToken, @LabelUri;

	WHILE (@@FETCH_STATUS=0) BEGIN
		
		-- Retrieve the RegistryLabel	
		SELECT @RegistryLabel = rgyRRtl.EntityLabel
			FROM pds_NResource AS RR INNER 
			JOIN pds_NResource AS rgyRR ON (RR.RecordRegistryInfosetGuidRef = rgyRR.InfosetGuidKey) INNER
			JOIN pds_NTagAndLabel AS entRRtl ON (RR.ResourceIidKey = entRRtl.ResourceIidRef) INNER
			JOIN pds_NTagAndLabel As rgyRRtl ON (rgyRR.ResourceIidKey = rgyRRtl.ResourceIidRef) AND (rgyRRtl.LabelIsCanonical = 1)
			WHERE (entRRtl.ResourceIidRef = @ResourceIid) AND (entRRtl.InternalKey = @InternalKey);
	
		IF (RIGHT(@RegistryLabel,1) <> '/') 
			SET @RegistryLabel = @RegistryLabel + '/';
		
		-- Always update the EntityLabel
		SELECT @EntityLabel = COALESCE(LabelUri, @RegistryLabel+LOWER(TagToken))
			FROM pds_NTagAndLabel WHERE (InternalKey = @InternalKey);
		
		-- failsafe to prevent nulls in uniquekey col	
		IF (@EntityLabel IS NULL) SET @EntityLabel = newid();

		UPDATE pds_NTagAndLabel SET EntityLabel = @EntityLabel
			WHERE (InternalKey = @InternalKey);	

		FETCH labels INTO @InternalKey, @ResourceIid, @TagToken, @LabelUri;
	
	END

	CLOSE labels
	DEALLOCATE labels

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NRegistryUpdateDependentEntityLabels')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceAccess]
(@ResourceIidKey int,
 @AgentIid int)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @IsAdmin bit, @IsEditor bit, @IsAuthor bit, @AccessCode int;
	SELECT @IsAdmin = AgentIsAdmin, @IsEditor = AgentIsEditor, @IsAuthor = AgentIsAuthor
		FROM pds_NAgent AS A WHERE A.AgentIidKey = @AgentIid;
		
	IF (@IsAuthor = 1) AND EXISTS(SELECT RecordHandle FROM pds_NResource AS R 
		WHERE (R.ResourceIidKey = @ResourceIidKey) AND (R.RecordManagedByAgentIidRef = @AgentIid))

		SELECT @AccessCode = 3;

	ELSE IF (@IsEditor = 1) AND EXISTS(SELECT RecordHandle FROM pds_NResource AS R
		INNER JOIN pds_NAgentRegistry AS E ON (R.RecordRegistryInfosetGuidRef = E.RegistryInfosetGuidRef)
		WHERE (R.ResourceIidKey = @ResourceIidKey) AND (E.AgentIidRef = @AgentIid))

		SELECT @AccessCode = 2;

	ELSE IF (@IsAdmin = 1)
	
		SELECT @AccessCode = 1;

	ELSE
	
		SELECT @AccessCode = -1;
	
	RETURN @AccessCode;

END
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceAccess')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceCheckEntityLabels](
@ResourceIidKey int,
@RecordRegistryGuidRef uniqueidentifier)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	DECLARE @RegistryLabel nvarchar(384);

		-- Retrieve the RegistryLabel for the NEW registry	
	SELECT @RegistryLabel = rgyRRtl.EntityLabel
		FROM pds_NResource AS rgyRR INNER
		JOIN pds_NTagAndLabel As rgyRRtl ON (rgyRR.ResourceIidKey = rgyRRtl.ResourceIidRef) AND (rgyRRtl.LabelIsCanonical = 1)
		WHERE (rgyRR.InfosetGuidKey = @RecordRegistryGuidRef);
	
	IF (RIGHT(@RegistryLabel,1) <> '/') SET @RegistryLabel = @RegistryLabel + '/';

	DECLARE @InternalKey int, @TagToken nvarchar(128), @LabelUri nvarchar(256), @EntityLabel nvarchar(384), 
		@count int, @exists int;

	DECLARE labels CURSOR FOR
		SELECT InternalKey, TagToken, LabelUri FROM pds_NTagAndLabel WHERE (ResourceIidRef = @ResourceIidKey);

	SET @exists = 0;
	OPEN labels;
	FETCH labels INTO @InternalKey, @TagToken, @LabelUri;

	WHILE (@@FETCH_STATUS=0) BEGIN
		
		-- create the new label
		SELECT @EntityLabel = COALESCE(LabelUri, @RegistryLabel+LOWER(TagToken))
			FROM pds_NTagAndLabel WHERE (InternalKey = @InternalKey);
		
		-- check existence of the new label
		SELECT @count = COUNT(*) FROM pds_NTagAndLabel WHERE (EntityLabel = @EntityLabel);
		IF (@count > 0) SET @exists = @exists - 1;	
		
		-- advance cursor
		FETCH labels INTO @InternalKey, @TagToken, @LabelUri;
	
	END;

	CLOSE labels;
	DEALLOCATE labels;
	
	RETURN @exists;

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceCheckEntityLabels')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceCountRecordsCreatedByAgent]
(@PdsAgentIid int)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @count int;

	SELECT @count = COUNT(ResourceIidKey) FROM pds_NResource
		WHERE (RecordCreatedByAgentIidRef = @PdsAgentIid);
	
	RETURN @count;

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceCountRecordsCreatedByAgent')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
/****** Object:  StoredProcedure [dbo].[gtg_PDRegistryDisp]    Script Date: 05/02/2008 19:44:23 ******/

CREATE PROCEDURE [dbo].[pds_NResourceEntityRegTagExists](
@EntityPrincipalTag nvarchar(128),
@RecordRegistryGuidRef uniqueidentifier = NULL,
@ResourceIidKey int = NULL,
@InternalKey int = NULL)
AS BEGIN

	/* provides existence check and returns existing label 
	       if registry/tag combo already exists */
	
	SET NOCOUNT ON
	
	-- @EntityPrincipalTag must be non-null to test for its existence
	IF @EntityPrincipalTag IS NULL RETURN -1;
	-- @RecordRegistryGuid and @ResourceIid must not be simultaneously null
	IF (@RecordRegistryGuidRef IS NULL) BEGIN
		IF (@ResourceIidKey IS NULL) BEGIN
			RETURN -2;
		END ELSE BEGIN
			SELECT @RecordRegistryGuidRef = RecordRegistryInfosetGuidRef
				FROM pds_NResource WHERE (ResourceIidKey = @ResourceIidKey);
			IF (@RecordRegistryGuidRef IS NULL) RETURN -3;
		END;	
	END;
	
	-- @EntityPrincipalTag and @RecordRegistryGuid should now both be non-null
	
	IF (@ResourceIidKey IS NULL) OR (@InternalKey IS NULL) BEGIN
		/* self does not yet exist; check all */
		SELECT T.InternalKey	
			FROM pds_NResource AS R INNER
			JOIN pds_NTagAndLabel AS T 
			ON (R.ResourceIidKey = T.ResourceIidRef)
			WHERE (R.RecordRegistryInfosetGuidRef = @RecordRegistryGuidRef)
			AND (LOWER(T.TagToken) = LOWER(@EntityPrincipalTag));

		RETURN ISNULL(@@ROWCOUNT,0);

	END ELSE BEGIN
		/* self already does exist; check all except self */
		SELECT T.InternalKey	
			FROM pds_NResource AS R INNER
			JOIN pds_NTagAndLabel AS T 
			ON (R.ResourceIidKey = T.ResourceIidRef)
			WHERE (R.RecordRegistryInfosetGuidRef = @RecordRegistryGuidRef)
			AND (LOWER(T.TagToken) = LOWER(@EntityPrincipalTag))
			AND (T.InternalKey <> @InternalKey);

		RETURN ISNULL(@@ROWCOUNT,0);

	END

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceEntityRegTagExists')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceGenerateRecordHandle]
(@Handle char(9) output)
AS
BEGIN

	SET NOCOUNT ON;

	DECLARE @nc tinyint, @rows tinyint, @loops tinyint;
	SET @nc = 9;

	EXEC pds_ARandomHexStrVarChar @nc, @Handle output;
	SELECT @rows = COUNT(ResourceIidKey) FROM pds_NResource WHERE RecordHandle = @Handle;
	-- (@rows > 0) implies @Handle already exists and thus is duplicate or replicate
	SET @loops = 1;
	WHILE (@rows > 0) AND (@loops < 99)
	BEGIN
		EXEC pds_ARandomHexStrVarChar @nc, @Handle output;	
		SELECT @rows = COUNT(ResourceIidKey) FROM pds_NResource WHERE RecordHandle = @Handle;
		SET @loops = @loops + 1;	
	END;
	-- (@rows = 0) implies new random @Handle is unique 
	-- (also indicates "no error" return value)
	RETURN @rows;

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceGenerateRecordHandle')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceInfosetValidateAtDoors](
@ResourceIidKey int,
@PdsAgentIid int,
@StatusAtDoors tinyint output)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @IsEditor bit, @IsAdmin bit;
	SELECT @IsEditor = AgentIsEditor, @IsAdmin = AgentIsAdmin 
		FROM pds_NAgent WHERE AgentIidKey = @PdsAgentIid;
		
	/* If Agent is not also (Editor or Admin) and does not have Management permissions, then exit */
	IF (@IsEditor = 0) AND (@IsAdmin = 0) AND NOT EXISTS(SELECT RecordHandle FROM pds_NResource
		WHERE (ResourceIidKey = @ResourceIidKey) AND (RecordManagedByAgentIidRef = @PdsAgentIid))
		RETURN -1;

	-- TODO: ? add code for validating all relevant DOORS fields other than Location ?

	-- validate existence of at least one resolveable URL 
	--		as either a Label URI that is also URL
	--		or else a Location that has a URL
	DECLARE @TypeCode smallint, @Count int, @LocationExists bit;
	-- check for existence of LocationURL
	SELECT @Count = COUNT(*) FROM pds_NLocation
		WHERE (ResourceIidRef = @ResourceIidKey) AND (LocationURL IS NOT NULL);
	-- if none, then check for existence of resolvable labels, URIs that are also URLs,
	--		but only for entities that are not components
	IF (@Count = 0) BEGIN
		SELECT @TypeCode = EntityTypeCode FROM pds_NResource
			WHERE (ResourceIidKey = @ResourceIidKey);
		-- check only entities that are not constituents (>=60) or else are untyped (=0)
		IF (@TypeCode >= 60) OR (@TypeCode = 0)
			SELECT @Count = COUNT(*) FROM pds_NTagAndLabel 
				WHERE (ResourceIidRef = @ResourceIidKey) AND (LabelIsResolvable = 1);
	END;
	
	IF (@Count > 0) SET @StatusAtDoors = 1
	ELSE SET @StatusAtDoors = 0;

	-- update main record with counts, status, datetime stamp, etc
	-- recall RecordUpdatedOn handled by UPDATE trigger so not necessary here
	-- TODO: re-evaluate re-factor updates in UPDATE trigger versus updates in this validate storproc
	-- TODO: necessary to assure that Validate timestamp >= Update timestamp ? 
	-- TODO: use GETUTCDATE + timeadvance ? else re-code via re-factor or shift to middle-layer
    UPDATE pds_NResource
	    SET InfosetDoorsStatusCode = @StatusAtDoors, 
			InfosetDoorsStatusTestedOn = GETUTCDATE(),
			RecordUpdatedByAgentIidRef = @PdsAgentIid
		WHERE (ResourceIidKey = @ResourceIidKey);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceInfosetValidateAtDoors')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceUpdateEntityLabels](
@ResourceIidKey int)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	DECLARE @RegistryLabel nvarchar(384);

		-- Retrieve the RegistryLabel	
		SELECT @RegistryLabel = rgyRRtl.EntityLabel
			FROM pds_NResource AS RR INNER 
			JOIN pds_NResource AS rgyRR ON (RR.RecordRegistryInfosetGuidRef = rgyRR.InfosetGuidKey) INNER
			JOIN pds_NTagAndLabel As rgyRRtl ON (rgyRR.ResourceIidKey = rgyRRtl.ResourceIidRef) AND (rgyRRtl.LabelIsCanonical = 1)
			WHERE (RR.ResourceIidKey = @ResourceIidKey);
	
		IF (RIGHT(@RegistryLabel,1) <> '/') 
			SET @RegistryLabel = @RegistryLabel + '/';

	DECLARE @InternalKey int, @TagToken nvarchar(128), @LabelUri nvarchar(256), @EntityLabel nvarchar(384);

	DECLARE labels CURSOR FOR
		SELECT InternalKey, TagToken, LabelUri FROM pds_NTagAndLabel WHERE (ResourceIidRef = @ResourceIidKey);

	OPEN labels;
	FETCH labels INTO @InternalKey, @TagToken, @LabelUri;

	WHILE (@@FETCH_STATUS=0) BEGIN
		
		-- Always update the EntityLabel
		SELECT @EntityLabel = COALESCE(LabelUri, @RegistryLabel+LOWER(TagToken))
			FROM pds_NTagAndLabel WHERE (InternalKey = @InternalKey);
		
		-- failsafe to prevent nulls in uniquekey col	
		IF (@EntityLabel IS NULL) SET @EntityLabel = newid();

		UPDATE pds_NTagAndLabel SET EntityLabel = @EntityLabel
			WHERE (InternalKey = @InternalKey);	

		FETCH labels INTO @InternalKey, @TagToken, @LabelUri;
	
	END;

	CLOSE labels;
	DEALLOCATE labels;

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceUpdateEntityLabels')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryDirectoryDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NSecondaryDirectory
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryDirectoryDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryDirectoryInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@SecondaryDirectoryGuidRef uniqueidentifier)
AS
BEGIN

	SET NOCOUNT ON;
	
	INSERT INTO pds_NSecondaryDirectory 
		(ResourceIidRef, Priority, DirectoryInfosetGuidRef) 
		VALUES 
		(@ResourceIidRef, @Priority, @SecondaryDirectoryGuidRef);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryDirectoryInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryDirectoryUpdate](
@Priority tinyint,
@SecondaryDirectoryGuidRef uniqueidentifier,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
	
	UPDATE pds_NSecondaryDirectory 
		SET Priority = @Priority,
		DirectoryInfosetGuidRef = @SecondaryDirectoryGuidRef
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryDirectoryUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryRegistryDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NSecondaryRegistry
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryRegistryDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryRegistryInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@SecondaryRegistryGuidRef uniqueidentifier)
AS
BEGIN

	SET NOCOUNT ON;
	
	INSERT INTO pds_NSecondaryRegistry
		(ResourceIidRef, Priority, RegistryInfosetGuidRef) 
		VALUES 
		(@ResourceIidRef, @Priority, @SecondaryRegistryGuidRef);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryRegistryInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryRegistryUpdate](
@Priority tinyint,
@SecondaryRegistryGuidRef uniqueidentifier,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
	
	UPDATE pds_NSecondaryRegistry 
		SET Priority = @Priority,
		RegistryInfosetGuidRef = @SecondaryRegistryGuidRef
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryRegistryUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingLabelDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NSupportingLabel
	    WHERE InternalKey = @InternalKey;
 
	RETURN ISNULL(@@ROWCOUNT,0);

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingLabelDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingLabelInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@SupportingLabel nvarchar(128),
@LabelIsRestricted bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
    
    INSERT INTO pds_NSupportingLabel
    (ResourceIidRef, Priority, SupportingLabel, LabelIsRestricted) 
    VALUES 
    (@ResourceIidRef, @Priority, @SupportingLabel, @LabelIsRestricted);

	RETURN ISNULL(@@ROWCOUNT,0);

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingLabelInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingLabelUpdate](
@Priority tinyint = 0,
@SupportingLabel nvarchar(128),
@LabelIsRestricted bit = 0,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
  
    UPDATE pds_NSupportingLabel
		SET Priority = @Priority, SupportingLabel = @SupportingLabel,
		LabelIsRestricted = @LabelIsRestricted
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingLabelUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingTagDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;

    DELETE FROM pds_NSupportingTag
	    WHERE InternalKey = @InternalKey;
 
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingTagDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingTagInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@SupportingTag nvarchar(128),
@TagIsRestricted bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
    
    INSERT INTO pds_NSupportingTag
    (ResourceIidRef, Priority, SupportingTag, TagIsRestricted) 
    VALUES 
    (@ResourceIidRef, @Priority, @SupportingTag, @TagIsRestricted);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingTagInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingTagUpdate](
@Priority tinyint = 0,
@SupportingTag nvarchar(128),
@TagIsRestricted bit = 0,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
  
    UPDATE pds_NSupportingTag
		SET Priority = @Priority, SupportingTag = @SupportingTag,
		TagIsRestricted = @TagIsRestricted
		WHERE InternalKey = @InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingTagUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelDelete](
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
	-- Disallow deletion of the canonical tag/label:
	--	implies that a different tag/label must first be set as canonical
	--	before the requested @InternalKey identified tag/label can be deleted
    DELETE FROM pds_NTagAndLabel
	    WHERE (InternalKey = @InternalKey) AND (LabelIsCanonical = 0);
 
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelInsert](
@ResourceIidRef int,
@Priority tinyint = 0,
@TagToken nvarchar(128),
@LabelUri nvarchar(256),
@LabelIsCanonical bit = 0,
@LabelIsPrivate bit = 0,
@LabelIsResolvable bit = 0,
@LabelIsTagIndependent bit = 0)
AS
BEGIN

	SET NOCOUNT ON;
    
    INSERT INTO pds_NTagAndLabel
    (ResourceIidRef, Priority, TagToken, LabelUri, 
     LabelIsTagIndependent, LabelIsCanonical, LabelIsPrivate, LabelIsResolvable) 
    VALUES 
    (@ResourceIidRef, @Priority, @TagToken, @LabelUri, 
     @LabelIsTagIndependent, @LabelIsCanonical, @LabelIsPrivate, @LabelIsResolvable);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelInsertAliasTagToken](
@ResourceIidRef int,
@TagToken nvarchar(128),
@Priority tinyint)
AS
BEGIN

	-- reset any priority of 0 to 255
	IF (@Priority <= 0) OR (256 <= @Priority) SET @Priority = 255;

	INSERT INTO pds_NTagAndLabel 
		(ResourceIidRef, Priority, TagToken, LabelIsResolvable) 
	VALUES 
	    (@ResourceIidRef, @Priority, @TagToken, 1);

	RETURN ISNULL(@@ROWCOUNT,0);

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelInsertAliasTagToken')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelInsertUpdatePrincipalTag](
@ResourceIidRef int,
@PrincipalTag nvarchar(128))
AS
BEGIN

	-- Principal tag defined to be the priority-ordered
	--		first tag with LabelIsCanonical = 1
	-- These Principal Tags assumed to be dependent on Registry Label
	--		so that LabelIsTagIndependent = 0 (the default for new records)
	-- Note that an integrity problem exists if there is 
	--		more than one record with LabelIsCanonical = 1
	
	SET NOCOUNT ON;
    
    DECLARE @CanonicalKey int, @NumFound int;
    
    SELECT TOP 1
		@CanonicalKey = InternalKey FROM pds_NTagAndLabel
		WHERE (ResourceIidRef = @ResourceIidRef) AND (LabelIsCanonical = 1)
		ORDER BY Priority, InternalKey;
		
	SET @NumFound = ISNULL(@@ROWCOUNT, 0);
		
	IF (@NumFound > 0) BEGIN
		
		UPDATE pds_NTagAndLabel
			SET TagToken = @PrincipalTag
			WHERE (InternalKey = @CanonicalKey);
	
	END ELSE BEGIN
		
	-- Require any new PrincipalTag to have 
	--		Priority = 0, LabelIsCanonical = 1, and LabelIsResolvable = 1
	   INSERT INTO pds_NTagAndLabel 
			(ResourceIidRef, Priority, TagToken, LabelIsCanonical, LabelIsResolvable, LabelIsTagIndependent) 
		VALUES 
		    (@ResourceIidRef, 0, @PrincipalTag, 1, 1, 0);
    
    END;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelInsertUpdatePrincipalTag')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelUpdate](
@Priority tinyint = 0,
@TagToken nvarchar(128),
@LabelUri nvarchar(256),
@LabelIsCanonical bit = 0,
@LabelIsPrivate bit = 0,
@LabelIsResolvable bit = 0,
@LabelIsTagIndependent bit = 0,
@InternalKey int)
AS
BEGIN

	SET NOCOUNT ON;
  
    UPDATE pds_NTagAndLabel
		SET Priority = @Priority, 
			TagToken = @TagToken, LabelUri = @LabelUri, 
			LabelIsCanonical = @LabelIsCanonical,
			LabelIsPrivate = @LabelIsPrivate,
			LabelIsResolvable = @LabelIsResolvable,
			LabelIsTagIndependent = @LabelIsTagIndependent
		WHERE (InternalKey = @InternalKey);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NCrossReferenceSelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;
  
    SELECT * FROM pds_NCrossReferenceView 
		WHERE (ResourceIidRef = @ResourceIidRef) 
		ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NCrossReferenceSelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NLocationSelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;

    SELECT * FROM pds_NLocationView
		WHERE (ResourceIidRef = @ResourceIidRef) 
		ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NLocationSelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NRegistryRestrictionXml]
(@RegistryGuid uniqueidentifier,
 @RestrictionXml xml output)
AS
BEGIN

	SET NOCOUNT ON;

	SET @RestrictionXml = 
	(SELECT COUNT(*) AS '@Count', 
	
		(SELECT COUNT(*) AS '@Count',	
			
			(SELECT 
				RestrictionAndIndex AS "@AndIndex",
				RestrictionOrIndex AS "@OrIndex", 
				Restriction AS "text()"
			FROM pds_NRegistryRestrictionView 
			WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 0) AND (RestrictionAndIndex = 0)
			ORDER BY RestrictionAndIndex, RestrictionOrIndex
			FOR XML PATH('WordPhrase'), TYPE),
			
			(SELECT 
				RestrictionAndIndex AS "@AndIndex",
				RestrictionOrIndex AS "@OrIndex", 
				Restriction AS "text()"
			FROM pds_NRegistryRestrictionView 
			WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 0) AND (RestrictionAndIndex > 0)
			ORDER BY RestrictionAndIndex, RestrictionOrIndex
			FOR XML PATH('WordStem'), TYPE)		
		
		FROM pds_NRegistryRestrictionView 
		WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 0)
		FOR XML PATH('ForSupportingTags'), TYPE),
	
		(SELECT COUNT(*) AS '@Count',
		
			(SELECT 
				RestrictionAndIndex AS "@AndIndex",
				RestrictionOrIndex AS "@OrIndex", 
				Restriction AS "text()"
			FROM pds_NRegistryRestrictionView 
			WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 1)
			ORDER BY RestrictionAndIndex, RestrictionOrIndex
			FOR XML PATH('ConceptLabel'), TYPE)
			
		FROM pds_NRegistryRestrictionView
		WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 1)
		FOR XML PATH('ForSupportingLabels'), TYPE)	
	
	FROM pds_NRegistryRestrictionView
	WHERE (InfosetGuidRef = @RegistryGuid)
	FOR XML PATH('RegistryRestrictions'), TYPE);
	
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NRegistryRestrictionXml')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NRegistryRestrictionXmlAlt]
(@RegistryGuid uniqueidentifier,
 @RestrictionXml xml output)
AS
BEGIN

	SET NOCOUNT ON;

	SET @RestrictionXml = 
	(SELECT COUNT(*) AS '@Count', 
	
		(SELECT COUNT(*) AS '@Count',	
			
			(SELECT 
				RestrictionAndIndex AS "@AndIndex",
				RestrictionOrIndex AS "@OrIndex", 
				Restriction AS "text()"
			FROM pds_NRegistryRestrictionView 
			WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 0)
			ORDER BY RestrictionAndIndex, RestrictionOrIndex
			FOR XML PATH('ForSupportingTag'), TYPE)
		
		FROM pds_NRegistryRestrictionView 
		WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 0)
		FOR XML PATH('ForSupportingTags'), TYPE),
	
		(SELECT COUNT(*) AS '@Count',
		
			(SELECT 
				RestrictionAndIndex AS "@AndIndex",
				RestrictionOrIndex AS "@OrIndex", 
				Restriction AS "text()"
			FROM pds_NRegistryRestrictionView 
			WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 1)
			ORDER BY RestrictionAndIndex, RestrictionOrIndex
			FOR XML PATH('ForSupportingLabel'), TYPE)
			
		FROM pds_NRegistryRestrictionView
		WHERE (InfosetGuidRef = @RegistryGuid) AND (RestrictionIsLabel = 1)
		FOR XML PATH('ForSupportingLabels'), TYPE)	
	
	FROM pds_NRegistryRestrictionView
	WHERE (InfosetGuidRef = @RegistryGuid)
	FOR XML PATH('RegistryRestrictions'), TYPE);
	
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NRegistryRestrictionXmlAlt')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
/****** Object:  StoredProcedure [dbo].[gtg_PDRegistryDisp]    Script Date: 05/02/2008 19:44:23 ******/

CREATE PROCEDURE [dbo].[pds_NResourceDelete](
@ResourceIidKey int,
@PdsAgentIid int)
AS BEGIN

	SET NOCOUNT ON;
	
	DECLARE @AccessCode int;
	EXECUTE @AccessCode = pds_NResourceAccess @ResourceIidKey, @PdsAgentIid;
	IF (@AccessCode <= 0) RETURN -1;

	DELETE FROM pds_NResource
		WHERE ResourceIidKey = @ResourceIidKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceDelete')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceEntityLabelToInfosetGuid]
(@EntityLabel nvarchar(384),
 @InfosetGuid uniqueidentifier = null output)
AS
BEGIN

	SET NOCOUNT ON;

	SET @EntityLabel = RTRIM(LTRIM(@EntityLabel));
	IF @EntityLabel IS NOT NULL 
		SELECT @InfosetGuid = InfosetGuidKey 
			FROM pds_NEntityLabelView WHERE (EntityLabel = LOWER(@EntityLabel));
			
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceEntityLabelToInfosetGuid')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingLabelSelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;

    SELECT * FROM pds_NSupportingLabelView 
	    WHERE (ResourceIidRef = @ResourceIidRef) 
	    ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingLabelSelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingLabelValidate](
@ResourceIidRef int,
@RecordRegistryGuidRef uniqueidentifier = NULL,
@NumRestrictions tinyint output,
@LabelsAreValid bit output)
AS
BEGIN

	/* use null default option for @RecordRegistryGuid to assure
	validation against actual RecordRegistryGuid stored in record
	rather than against some arbitrarily different input guid  */

	SET NOCOUNT ON;

	DECLARE @EntityTypeCode smallint, @EntityOtherGuid uniqueidentifier, @EntityOtherRecRegGuid uniqueidentifier, @RetVal tinyint;
	SET @RetVal = 0;
	
	-- Get the @RecordRegistryGuid if necessary, also other fields needed
	IF (@RecordRegistryGuidRef IS NULL)
		SELECT @RecordRegistryGuidRef = RecordRegistryInfosetGuidRef, 
			@EntityTypeCode = EntityTypeCode, @EntityOtherGuid = EntityOtherInfosetGuidRef
			FROM pds_NResource	WHERE (ResourceIidKey = @ResourceIidRef)
	ELSE
		SELECT @EntityTypeCode = EntityTypeCode, @EntityOtherGuid = EntityOtherInfosetGuidRef
			FROM pds_NResource	WHERE (ResourceIidKey = @ResourceIidRef);
	
	-- Check existence of label restrictions from the Resource's Registry using input @RecordRegistryGuidRef
	SELECT @NumRestrictions = COUNT(*) FROM pds_NRegistryRestrictionView 
		WHERE (InfosetGuidRef = @RecordRegistryGuidRef) AND (RestrictionIsLabel = 1);
	IF (@NumRestrictions = 0) 
	BEGIN
		SET @LabelsAreValid = 1; -- valid by definition when NO restrictions exist for the registry
		RETURN @RetVal;
	END;
	
	-- Check existence of EntityOther and its Registry for MetaResource EntityTypeCode = 99
	IF (@EntityTypeCode = 99)
	BEGIN
		SELECT @EntityOtherRecRegGuid = RecordRegistryInfosetGuidRef
			FROM pds_NResource WHERE (InfosetGuidKey = @EntityOtherGuid);
		IF (@EntityOtherRecRegGuid = @RecordRegistryGuidRef) -- entity itself and entity other in same registry
		BEGIN
			SET @LabelsAreValid = 1; -- valid by definition when MetaResource record is in same registry as described record
			RETURN @RetVal;
		END; 
	END;
	
	-- Get the required label Restrictions from the Resource's Registry using input @RecordRegistryGuidRef
	DECLARE @Restrictions table (AndIndex tinyint, OrIndex tinyint, Restriction nvarchar(384));
	INSERT INTO @Restrictions (AndIndex, OrIndex, Restriction) 
		SELECT RestrictionAndIndex, RestrictionOrIndex, Restriction FROM pds_NRegistryRestrictionView
		WHERE (InfosetGuidRef = @RecordRegistryGuidRef) AND (RestrictionIsLabel = 1)
		ORDER BY RestrictionAndIndex, RestrictionOrIndex;
		
	-- Get the SupportingLabels for the Resource using input @ResourceIid
	--		but only those SupportingLabels where LabelIsRestricted is true;
	--		also note SupportingLabel is nvarchar(256)
	DECLARE @TestLabels TABLE (LabelUri nvarchar(256)); 
	DECLARE @NumTestLabels int, @NumMatches int;
	INSERT INTO @TestLabels (LabelUri) 
		SELECT LOWER(SupportingLabel) FROM pds_NSupportingLabel
			WHERE (ResourceIidRef = @ResourceIidRef) AND (LabelIsRestricted = 1);
	SELECT @NumTestLabels = COUNT(*) FROM @TestLabels WHERE (LabelUri IS NOT NULL);
	
	-- initialize @LabelsAreValid to false for remainder of algorithm
	SET @LabelsAreValid = 0;
	-- If there are label restrictions (@NumRestrictions > 0) but no test labels (@NumTestLabels = 0), 
	--   then return invalid else continue with test
	IF (@NumRestrictions > 0) AND (@NumTestLabels = 0) RETURN @RetVal;

	DECLARE @AndValid bit, @AndIndex tinyint, @AndIdxMax tinyint, @AndIdxNum tinyint;
	DECLARE @OrValid bit, @OrIndex tinyint, @OrIdxMax tinyint, @OrIdxNum tinyint;
	-- Algorithm assumes convention that AndIndex = 0, OrIndex = * lists phrases with multiple concepts per OrIndex
	--	 whereas AndIndex >= 1, OrIndex = * lists word stems with one concept per AndIndex	
	-- For AndIndex = 0, initialize false until proven true and if true then return else continue test for AndIndex >= 1
	SET @AndIndex = 0;
	SELECT @OrIdxNum = COUNT(*) FROM @Restrictions WHERE AndIndex = @AndIndex;
	IF (@OrIdxNum > 0) BEGIN
		-- Algorithm assumes @OrValid false until proven true (search loop until "accept" at least one "or")
		-- Algorithm assumes that OrIndex numbered sequentially beginning with 1,2,3,...
		SET @OrValid = 0;
		SET @OrIndex = 1;
		SELECT @OrIdxMax = MAX(OrIndex) FROM @Restrictions WHERE AndIndex = @AndIndex;
		WHILE (@OrValid = 0) AND (@OrIndex <= @OrIdxMax) BEGIN
			SELECT @NumMatches = COUNT(*) FROM @TestLabels WHERE LabelUri LIKE 
				(SELECT Restriction FROM @Restrictions WHERE AndIndex = @AndIndex AND OrIndex = @OrIndex);
			IF (@NumMatches > 0) SET @OrValid = 1; -- at least one "OR" has been found so accept (forces exit from inner OR while loop)
			SET @OrIndex = @OrIndex + 1; -- increment OR index for inner OR while loop
		END;
		IF (@OrValid = 1) BEGIN
			SET @LabelsAreValid = 1;
			RETURN @RetVal;
		END;
	END;	
	
	-- For AndIndex >= 1, re-initialize true until proven false (search loop until "reject" at least one "and")
	-- Algorithm assumes that AndIndex numbered sequentially beginning with 1,2,3,...	
	SELECT @AndIdxMax = MAX(AndIndex) FROM @Restrictions;
	IF (@AndIdxMax > 0) BEGIN
		SET @AndValid = 1;
		SET @AndIndex = 1;
		WHILE (@AndValid = 1) AND (@AndIndex <= @AndIdxMax) BEGIN	
		-- Algorithm assumes @OrValid false until proven true (search loop until "accept" at least one "or")
		-- Algorithm assumes that OrIndex numbered sequentially beginning with 1,2,3,...
			SET @OrValid = 0;
			SET @OrIndex = 1;
			SELECT @OrIdxMax = MAX(OrIndex) FROM @Restrictions WHERE AndIndex = @AndIndex;
			WHILE (@OrValid = 0) AND (@OrIndex <= @OrIdxMax) BEGIN
				SELECT @NumMatches = COUNT(*) FROM @TestLabels WHERE LabelUri LIKE 
					(SELECT Restriction FROM @Restrictions WHERE AndIndex = @AndIndex AND OrIndex = @OrIndex);
				IF (@NumMatches > 0) SET @OrValid = 1; -- at least one "OR" has been found so accept (forces exit from inner OR while loop)
				SET @OrIndex = @OrIndex + 1; -- increment OR index for inner OR while loop
			END;
			IF (@OrValid = 0) SET @AndValid = 0; -- at least one "AND" has not been found so reject (forces exist from outer AND while loop)
			SET @AndIndex = @AndIndex + 1; -- increment AND index for outer AND while loop
		END;
		IF (@AndValid = 1) BEGIN
			SET @LabelsAreValid = 1;
			RETURN @RetVal;
		END; 	
	END;
	
	-- no error return code
	RETURN @RetVal;

 END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingLabelValidate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingTagSelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;

    SELECT * FROM pds_NSupportingTagView 
	    WHERE (ResourceIidRef = @ResourceIidRef) 
	    ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingTagSelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSupportingTagValidate](
@ResourceIidRef int,
@RecordRegistryGuidRef uniqueidentifier = NULL,
@NumRestrictions tinyint output,
@TagsAreValid bit output)
AS
BEGIN

	/* use null default option for @RecordRegistryGuid to assure
	validation against actual RecordRegistryGuid stored in record
	rather than against some arbitrarily different input guid  */

	SET NOCOUNT ON;

	DECLARE @EntityTypeCode smallint, @EntityOtherGuid uniqueidentifier, @EntityOtherRecRegGuid uniqueidentifier, @RetVal tinyint;
	SET @RetVal = 0;
	
	-- Get the @RecordRegistryGuid if necessary, also other fields needed
	IF (@RecordRegistryGuidRef IS NULL)
		SELECT @RecordRegistryGuidRef = RecordRegistryInfosetGuidRef, 
			@EntityTypeCode = EntityTypeCode, @EntityOtherGuid = EntityOtherInfosetGuidRef
			FROM pds_NResource	WHERE (ResourceIidKey = @ResourceIidRef)
	ELSE
		SELECT @EntityTypeCode = EntityTypeCode, @EntityOtherGuid = EntityOtherInfosetGuidRef
			FROM pds_NResource	WHERE (ResourceIidKey = @ResourceIidRef);
	
	-- Check existence of tag restrictions from the Resource's Registry using input @RecordRegistryGuidRef
	SELECT @NumRestrictions = COUNT(*) FROM pds_NRegistryRestrictionView 
		WHERE (InfosetGuidRef = @RecordRegistryGuidRef) AND (RestrictionIsLabel = 0);
	IF (@NumRestrictions = 0) 
	BEGIN
		SET @TagsAreValid = 1; -- valid by definition when NO restrictions exist for the registry
		RETURN @RetVal;
	END;
	
	-- Check existence of EntityOther and its Registry for MetaResource EntityTypeCode = 99
	IF (@EntityTypeCode = 99)
	BEGIN
		SELECT @EntityOtherRecRegGuid = RecordRegistryInfosetGuidRef
			FROM pds_NResource WHERE (InfosetGuidKey = @EntityOtherGuid);
		IF (@EntityOtherRecRegGuid = @RecordRegistryGuidRef) -- entity itself and entity other in same registry
		BEGIN
			SET @TagsAreValid = 1; -- valid by definition when MetaResource record is in same registry as described record
			RETURN @RetVal;
		END; 
	END;
	
	-- Get the required tag Restrictions from the Resource's Registry using input @RecordRegistryGuidRef
	DECLARE @Restrictions table (AndIndex tinyint, OrIndex tinyint, Restriction nvarchar(256));
	INSERT INTO @Restrictions (AndIndex, OrIndex, Restriction) 
		SELECT RestrictionAndIndex, RestrictionOrIndex, Restriction FROM pds_NRegistryRestrictionView
		WHERE (InfosetGuidRef = @RecordRegistryGuidRef) AND (RestrictionIsLabel = 0)
		ORDER BY RestrictionAndIndex, RestrictionOrIndex;
		
	-- Get the EntityName, EntityNature and SupportingTags for the Resource using input @ResourceIidRef
	--		but only those SupportingTags where TagIsRestricted is true;
	--		also note SupportingTag is nvarchar(128) while EntityNature is nvarchar(256)
	DECLARE @TestTags TABLE (TagToken nvarchar(256)); 
	DECLARE @NumTestTags int, @NumMatches int;
	INSERT INTO @TestTags (TagToken)
		SELECT LOWER(EntityName) FROM pds_NResource
			WHERE (ResourceIidKey = @ResourceIidRef);
	INSERT INTO @TestTags (TagToken)
		SELECT LOWER(EntityNature) FROM pds_NResource
			WHERE (ResourceIidKey = @ResourceIidRef);
	INSERT INTO @TestTags (TagToken) 
		SELECT LOWER(SupportingTag) FROM pds_NSupportingTag
			WHERE (ResourceIidRef = @ResourceIidRef) AND (TagIsRestricted = 1);
	SELECT @NumTestTags = COUNT(*) FROM @TestTags WHERE (TagToken IS NOT NULL);
	
	-- initialize @TagsAreValid to false for remainder of algorithm
	SET @TagsAreValid = 0;
	-- If there are tag restrictions (@NumRestrictions > 0) but no test tags (@NumTestTags = 0), 
	--   then return invalid else continue with test
	IF (@NumRestrictions > 0) AND (@NumTestTags = 0) RETURN @RetVal;

	DECLARE @AndValid bit, @AndIndex tinyint, @AndIdxMax tinyint, @AndIdxNum tinyint;
	DECLARE @OrValid bit, @OrIndex tinyint, @OrIdxMax tinyint, @OrIdxNum tinyint;
	-- Algorithm assumes convention that AndIndex = 0, OrIndex = * lists phrases with multiple concepts per OrIndex
	--	 whereas AndIndex >= 1, OrIndex = * lists word stems with one concept per AndIndex	
	-- For AndIndex = 0, initialize false until proven true and if true then return else continue test for AndIndex >= 1
	SET @AndIndex = 0;
	SELECT @OrIdxNum = COUNT(*) FROM @Restrictions WHERE AndIndex = @AndIndex;
	IF (@OrIdxNum > 0) BEGIN
		-- Algorithm assumes @OrValid false until proven true (search loop until "accept" at least one "or")
		-- Algorithm assumes that OrIndex numbered sequentially beginning with 1,2,3,...
		SET @OrValid = 0;
		SET @OrIndex = 1;
		SELECT @OrIdxMax = MAX(OrIndex) FROM @Restrictions WHERE AndIndex = @AndIndex;
		WHILE (@OrValid = 0) AND (@OrIndex <= @OrIdxMax) BEGIN
			SELECT @NumMatches = COUNT(*) FROM @TestTags WHERE TagToken LIKE 
				(SELECT Restriction FROM @Restrictions WHERE AndIndex = @AndIndex AND OrIndex = @OrIndex);
			IF (@NumMatches > 0) SET @OrValid = 1; -- at least one "OR" has been found so accept (forces exit from inner OR while loop)
			SET @OrIndex = @OrIndex + 1; -- increment OR index for inner OR while loop
		END;
		IF (@OrValid = 1) BEGIN
			SET @TagsAreValid = 1;
			RETURN @RetVal;
		END;
	END;	
	
	-- For AndIndex >= 1, re-initialize true until proven false (search loop until "reject" at least one "and")
	-- Algorithm assumes that AndIndex numbered sequentially beginning with 1,2,3,...	
	SELECT @AndIdxMax = MAX(AndIndex) FROM @Restrictions;
	IF (@AndIdxMax > 0) BEGIN
		SET @AndValid = 1;
		SET @AndIndex = 1;
		WHILE (@AndValid = 1) AND (@AndIndex <= @AndIdxMax) BEGIN	
		-- Algorithm assumes @OrValid false until proven true (search loop until "accept" at least one "or")
		-- Algorithm assumes that OrIndex numbered sequentially beginning with 1,2,3,...
			SET @OrValid = 0;
			SET @OrIndex = 1;
			SELECT @OrIdxMax = MAX(OrIndex) FROM @Restrictions WHERE AndIndex = @AndIndex;
			WHILE (@OrValid = 0) AND (@OrIndex <= @OrIdxMax) BEGIN
				SELECT @NumMatches = COUNT(*) FROM @TestTags WHERE TagToken LIKE 
					(SELECT Restriction FROM @Restrictions WHERE AndIndex = @AndIndex AND OrIndex = @OrIndex);
				IF (@NumMatches > 0) SET @OrValid = 1; -- at least one "OR" has been found so accept (forces exit from inner OR while loop)
				SET @OrIndex = @OrIndex + 1; -- increment OR index for inner OR while loop
			END;
			IF (@OrValid = 0) SET @AndValid = 0; -- at least one "AND" has not been found so reject (forces exist from outer AND while loop)
			SET @AndIndex = @AndIndex + 1; -- increment AND index for outer AND while loop
		END;
		IF (@AndValid = 1) BEGIN
			SET @TagsAreValid = 1;
			RETURN @RetVal;
		END; 	
	END;
	
	-- no error return code
	RETURN @RetVal;

 END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSupportingTagValidate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NTagAndLabelSelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;

    SELECT * FROM pds_NTagAndLabelView 
	    WHERE (ResourceIidRef = @ResourceIidRef) 
	    ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NTagAndLabelSelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NRegistryUpdateRestrictionXml]
AS
BEGIN

	SET NOCOUNT ON;

	DECLARE @RegGuid uniqueidentifier, @RegXml xml;

	DECLARE registries CURSOR FOR
		SELECT InfosetGuidKey FROM pds_NResource 
		WHERE EntityTypeCode = 11 ORDER BY EntityName;

	OPEN registries
	FETCH registries INTO @RegGuid;

	WHILE (@@FETCH_STATUS=0) BEGIN
			
		EXEC pds_NRegistryRestrictionXml @RegGuid, @RegXml output;

		UPDATE S SET 
			S.EntityOtherMetadata = @RegXml,
			S.RecordUpdatedOn = getutcdate() 
			FROM pds_NResource AS S WHERE (S.InfosetGuidKey = @RegGuid);	

		FETCH registries INTO @RegGuid;
	
	END

	CLOSE registries;
	DEALLOCATE registries;

END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NRegistryUpdateRestrictionXml')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceInfosetValidateAtPortal](
@ResourceIidKey int,
@PdsAgentIid int,
@StatusAtPortal tinyint output)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @IsEditor bit, @IsAdmin bit;
	SELECT @IsEditor = AgentIsEditor, @IsAdmin = AgentIsAdmin 
		FROM pds_NAgent WHERE AgentIidKey = @PdsAgentIid;
		
	/* If Agent is not also (Editor or Admin) and does not have Management permissions, then exit */
	IF (@IsEditor = 0) AND (@IsAdmin = 0) AND NOT EXISTS(SELECT RecordHandle FROM pds_NResource
		WHERE (ResourceIidKey = @ResourceIidKey) AND (RecordManagedByAgentIidRef = @PdsAgentIid))
		RETURN -1;

	-- TODO: ? add code for validating all relevant PORTAL fields other than Restricted Tags/Labels ?
	-- else assume NO restrictions, valid by definition, and initialize as such

	-- compute @StatusAtPortal by testing Restricted Tags/Labels
	DECLARE @NumTagRestrictions tinyint, @TagsAreValid bit, @NumLabelRestrictions tinyint, @LabelsAreValid bit;
	EXEC pds_NSupportingTagValidate @ResourceIidKey, NULL, @NumTagRestrictions output, @TagsAreValid output;
	EXEC pds_NSupportingLabelValidate @ResourceIidKey, NULL, @NumLabelRestrictions output, @LabelsAreValid output;
	IF (@NumTagRestrictions IS NULL) OR (@NumLabelRestrictions IS NULL) RETURN -2;
	IF (@TagsAreValid IS NULL) OR (@LabelsAreValid IS NULL) RETURN -3;
	
	SET @StatusAtPortal = 2;
	IF (@NumTagRestrictions > 0) AND (@NumLabelRestrictions > 0) 
		BEGIN
			-- either/or here assumes that they are harmonized!!! rather than independent
			IF (@TagsAreValid = 1 OR @LabelsAreValid = 1) SET @StatusAtPortal = 1
			ELSE SET @StatusAtPortal = 0
		END 
	ELSE IF (@NumTagRestrictions > 0) 
		BEGIN
			IF (@TagsAreValid = 1) SET @StatusAtPortal = 1
			ELSE SET @StatusAtPortal = 0
		END
	ELSE IF (@NumLabelRestrictions > 0) 
		BEGIN
			IF (@LabelsAreValid = 1) SET @StatusAtPortal = 1
			ELSE SET @StatusAtPortal = 0		
		END
	ELSE -- no restrictions, valid by definition
		SET @StatusAtPortal = 1; 
	
	-- update main record with counts, status, datetime stamp, etc
	-- recall RecordUpdatedOn handled by UPDATE trigger so not necessary here
	-- TODO: re-evaluate re-factor updates in UPDATE trigger versus updates in this validate storproc
	-- TODO: necessary to assure that Validate timestamp >= Update timestamp ? 
	-- TODO: use GETUTCDATE + timeadvance ? else re-code via re-factor or shift to middle-layer
    UPDATE pds_NResource
	    SET InfosetPortalStatusCode = @StatusAtPortal, 
			InfosetPortalStatusTestedOn = GETUTCDATE(),
			RecordUpdatedByAgentIidRef = @PdsAgentIid
		WHERE (ResourceIidKey = @ResourceIidKey);

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceInfosetValidateAtPortal')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceInsert](
@EntityTypeCode smallint = 0, 
@EntityName nvarchar(128) = null, 
@EntityNature nvarchar(256) = null,
@EntityOtherMetadata xml = null,
@EntityDescription xml = null,
@EntityOtherLabel nvarchar(384) = null,
@RecordRegistrarGuidRef uniqueidentifier = 'AB876671-9D8E-4352-A1FC-35647CBDB225',
@RecordRegistryGuidRef uniqueidentifier = 'A5BAE24E-E796-488D-B955-763D85818150', 
@RecordDirectoryGuidRef uniqueidentifier = '15E055A1-F05F-4054-ABF5-3DA9599D63FF', 
@RecordHandle char(9) output,
@InfosetPortalStatusCode smallint = 2,
@InfosetDoorsStatusCode smallint = 2,
@PdsAgentIid int)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @exists int, @EntityOtherGuidRef uniqueidentifier;

	-- allow non-null EntityOtherLabel/Guid only for an Infoset with TypeCode = 99
	SET @EntityOtherGuidRef = NULL; -- initialize
	IF (@EntityOtherLabel IS NOT NULL) AND (@EntityTypeCode <> 99)
		SET @EntityOtherLabel = NULL;
	IF (@EntityOtherLabel IS NOT NULL) 
		EXEC pds_NResourceEntityLabelToInfosetGuid @EntityOtherLabel, @EntityOtherGuidRef output;
	-- create random character handle for record	
	EXEC @exists = pds_NResourceGenerateRecordHandle @RecordHandle output;
	IF (@exists > 0) RETURN -1;
	
    INSERT INTO pds_NResource 
		(EntityTypeCode, EntityName, EntityNature, EntityOtherMetadata, EntityDescription, EntityOtherInfosetGuidRef,
		 RecordRegistrarInfosetGuidRef, RecordRegistryInfosetGuidRef, RecordDirectoryInfosetGuidRef, 
		 RecordHandle, RecordCreatedByAgentIidRef, RecordUpdatedByAgentIidRef, RecordManagedByAgentIidRef,
		 InfosetPortalStatusCode, InfosetDoorsStatusCode) 
		VALUES 
		(@EntityTypeCode, @EntityName, @EntityNature, @EntityOtherMetadata, @EntityDescription, @EntityOtherGuidRef,
		 @RecordRegistrarGuidRef, @RecordRegistryGuidRef, @RecordDirectoryGuidRef, 
		 @RecordHandle, @PdsAgentIid, @PdsAgentIid, @PdsAgentIid,
		 @InfosetPortalStatusCode, @InfosetDoorsStatusCode);

	RETURN ISNULL(@@ROWCOUNT,0);
	
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceInsert')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceSelectByRegistry]
(@SetRegistryGuid uniqueidentifier = '00000000-0000-0000-0000-000000000000',
 @SetTypeCode smallint = -1,
 @PdsAgentIid int = -1,
 @ByAuthor bit = 0,
 @ByEditor bit = 0,
 @ByAdmin bit = 0)
AS
BEGIN

	/* TODO: complete coding for RecordRegistryGuid for registrars,registries,directories */
	SET NOCOUNT ON;
	
	DECLARE @ByRegistry bit;
	SET @SetRegistryGuid = ISNULL(@SetRegistryGuid,'00000000-0000-0000-0000-000000000000');
	/* use cast, convert or implicit conversion for char to uniqueidentifier */
	IF (@SetRegistryGuid = '00000000-0000-0000-0000-000000000000')
		SET @ByRegistry = 0;
	ELSE
		SET @ByRegistry = 1;
		
	SET @SetTypeCode = ISNULL(@SetTypeCode,-1);
	SET @PdsAgentIid = ISNULL(@PdsAgentIid,-1);

	IF (@SetTypeCode <= 0) AND (@ByAuthor = 0) 
		AND (@ByEditor = 0) AND (@ByAdmin = 0) BEGIN

		IF (@SetTypeCode = -1) BEGIN -- any and ALL resources SetType -1

			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 0)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 0)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -2) BEGIN -- all constituents

			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 40) AND (EntityTypeCode <= 59)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 0)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -3) BEGIN -- all components

			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 1) AND (EntityTypeCode <= 39)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode >= 0)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = 0) BEGIN -- UNTYPED type 0
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 0)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 0)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -40) BEGIN -- ORGANIZATIONS type 40
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 40)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 40)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -50) BEGIN -- PERSONS type 50
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 50)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 50)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;
				
		END ELSE IF (@SetTypeCode = -60) BEGIN -- OFFLINE type 60
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 60)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 60)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -70) BEGIN -- ONLINE type 70
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 70)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 70)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -99) BEGIN -- INFOSET type 99
			
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 99)
					AND (InfosetIsPrivate = 0)
					ORDER BY EntityName;
			ELSE
				SELECT * FROM pds_NResourceView
					WHERE (EntityTypeCode = 99)
					AND (InfosetIsPrivate = 0)
					AND (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -10) BEGIN -- REGISTRIES type 10-12
			
			SELECT * FROM pds_NResourceView
				WHERE (EntityTypeCode >= 10) AND (EntityTypeCode <= 12)
				AND (InfosetIsPrivate = 0)
				ORDER BY EntityName;

		END ELSE IF (@SetTypeCode = -20) BEGIN -- DIRECTORIES type 20-22
			
			SELECT * FROM pds_NResourceView
				WHERE (EntityTypeCode >= 20) AND (EntityTypeCode <= 22)
				AND (InfosetIsPrivate = 0)
				ORDER BY EntityName;

 		END ELSE IF (@SetTypeCode = -30) BEGIN -- REGISTRARS type 30-32
			
			SELECT * FROM pds_NResourceView
				WHERE (EntityTypeCode >= 30) AND (EntityTypeCode <= 32)
				AND (InfosetIsPrivate = 0)
				ORDER BY EntityName;
				
		END;
	
	END ELSE IF (@PdsAgentIid > 0) BEGIN
	
		DECLARE @IsAdmin bit, @IsEditor bit, @IsAuthor bit;
		SELECT @IsAdmin = AgentIsAdmin, 
			@IsEditor = AgentIsEditor, 
			@IsAuthor = AgentIsAuthor
			FROM pds_NAgent AS A WHERE A.AgentIidKey = @PdsAgentIid;
		
		IF (@IsAdmin = 1) AND (@ByAdmin = 1) BEGIN
		
			IF (@ByRegistry = 0) BEGIN
				IF (@SetTypeCode = -3) 
					SELECT * FROM pds_NResourceView
						WHERE (EntityTypeCode >= 1) AND (EntityTypeCode <= 39)
						ORDER BY EntityName;
				ELSE
					SELECT * FROM pds_NResourceView
						ORDER BY EntityName;
			END ELSE BEGIN
				SELECT * FROM pds_NResourceView
					WHERE (RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY EntityName;
			END

		END ELSE IF (@IsEditor = 1) AND (@ByEditor = 1) BEGIN
		
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView AS R
					WHERE ( R.RecordManagedByAgentIidRef = @PdsAgentIid)
					OR (R.RecordRegistryGuidRef IN (SELECT E.RegistryInfosetGuidRef 
						FROM pds_NAgentRegistry AS E 
						WHERE E.AgentIidRef = @PdsAgentIid) 
					AND R.EntityTypeCode IN (SELECT C.CodeKey 
						FROM pds_AEntityTypeEnum AS C
						WHERE C.TypeEditedByEditor = 1)) 
					ORDER BY R.EntityName;
			ELSE
				SELECT * FROM pds_NResourceView AS R
					WHERE (R.RecordRegistryGuidRef = @SetRegistryGuid)
					AND ((R.RecordManagedByAgentIidRef = @PdsAgentIid)
					OR (R.RecordRegistryGuidRef IN (SELECT E.RegistryInfosetGuidRef 
							FROM pds_NAgentRegistry AS E 
							WHERE E.AgentIidRef = @PdsAgentIid) 
					AND R.EntityTypeCode IN (SELECT C.CodeKey 
						FROM pds_AEntityTypeEnum AS C
						WHERE C.TypeEditedByEditor = 1))) 
					ORDER BY R.EntityName;
	
		END ELSE IF (@IsAuthor = 1) AND (@ByAuthor = 1) BEGIN
		
			IF (@ByRegistry = 0) 
				SELECT * FROM pds_NResourceView AS R
					WHERE (R.RecordManagedByAgentIidRef = @PdsAgentIid)
					AND (R.EntityTypeCode IN (SELECT C.CodeKey 
						FROM pds_AEntityTypeEnum AS C
						WHERE C.TypeEditedByAuthor = 1) )
					ORDER BY R.EntityName;
			ELSE
				SELECT * FROM pds_NResourceView AS R
					WHERE (R.RecordManagedByAgentIidRef = @PdsAgentIid)
					AND (R.EntityTypeCode IN (SELECT C.CodeKey 
						FROM pds_AEntityTypeEnum AS C
						WHERE C.TypeEditedByAuthor = 1) )
					AND (R.RecordRegistryGuidRef = @SetRegistryGuid)
					ORDER BY R.EntityName;

		END;

	END;
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceSelectByRegistry')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NResourceUpdate](
@ResourceIidKey int,
@EntityTypeCode smallint = 0, 
@EntityName nvarchar(128) = null, 
@EntityNature nvarchar(256) = null,
@EntityOtherMetadata xml = null,
@EntityDescription xml = null,
@EntityOtherLabel nvarchar(384) = null,
@RecordRegistrarGuidRef uniqueidentifier = 'AB876671-9D8E-4352-A1FC-35647CBDB225',
@RecordRegistryGuidRef uniqueidentifier = 'A5BAE24E-E796-488D-B955-763D85818150', 
@RecordDirectoryGuidRef uniqueidentifier = '15E055A1-F05F-4054-ABF5-3DA9599D63FF', 
@InfosetPortalStatusCode smallint = 2,
@InfosetDoorsStatusCode smallint = 2,
@PdsAgentIid int)
AS
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @AccessCode int, @exists int, @EntityOtherGuidRef uniqueidentifier,
		@OldRegistryGuidRef uniqueidentifier, @UpdateLabels bit;
	EXECUTE @AccessCode = pds_NResourceAccess @ResourceIidKey, @PdsAgentIid;
	IF (@AccessCode <= 0) RETURN -1;

	-- allow non-null EntityOtherLabel/Guid only for an Infoset with TypeCode = 99
	SET @EntityOtherGuidRef = NULL; -- initialize
	IF (@EntityOtherLabel IS NOT NULL) AND (@EntityTypeCode <> 99)
		SET @EntityOtherLabel = NULL;
	IF (@EntityOtherLabel IS NOT NULL) 
		EXEC pds_NResourceEntityLabelToInfosetGuid @EntityOtherLabel, @EntityOtherGuidRef output;
		
	-- check for change in registry
	SELECT @OldRegistryGuidRef = RecordRegistryInfosetGuidRef 
		FROM pds_NResource WHERE (ResourceIidKey = @ResourceIidKey);
	IF (@OldRegistryGuidRef = @RecordRegistryGuidRef) 
		SET @UpdateLabels = 0
	ELSE
		SET @UpdateLabels = 1;
	
	-- check for ability to move labels from old to new registry	
	IF (@UpdateLabels = 1) BEGIN
		EXECUTE @exists = pds_NResourceCheckEntityLabels @ResourceIidKey, @RecordRegistryGuidRef;
		IF (@exists < 0) BEGIN
			SET @UpdateLabels = 0;
			SET @RecordRegistryGuidRef = @OldRegistryGuidRef;
		END;
	END;
		
	-- TODO: check prior existence of PTags under change of registry
	-- otherwise must either prevent the registry change or delete conflicting PTags	

    UPDATE pds_NResource 
	    SET EntityTypeCode = @EntityTypeCode, 
			EntityName = @EntityName, EntityNature = @EntityNature, 
			EntityOtherMetadata = @EntityOtherMetadata, EntityDescription = @EntityDescription,
			EntityOtherInfosetGuidRef = @EntityOtherGuidRef,
			RecordRegistrarInfosetGuidRef = @RecordRegistrarGuidRef, 
			RecordRegistryInfosetGuidRef = @RecordRegistryGuidRef, 
			RecordDirectoryInfosetGuidRef = @RecordDirectoryGuidRef, 
			InfosetPortalStatusCode = @InfosetPortalStatusCode, 
			InfosetDoorsStatusCode = @InfosetDoorsStatusCode,
			RecordUpdatedByAgentIidRef = @PdsAgentIid
		WHERE (ResourceIidKey = @ResourceIidKey);

	IF (@UpdateLabels = 1) BEGIN
		EXECUTE pds_NResourceUpdateEntityLabels @ResourceIidKey;
	END;
	
	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NResourceUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryDirectorySelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;
    
    SELECT * FROM pds_NSecondaryDirectoryView 
		WHERE (ResourceIidRef = @ResourceIidRef)
	    ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryDirectorySelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE PROCEDURE [dbo].[pds_NSecondaryRegistrySelect](
@ResourceIidRef int)
AS
BEGIN

	SET NOCOUNT ON;
    
    SELECT * FROM pds_NSecondaryRegistryView 
		WHERE (ResourceIidRef = @ResourceIidRef)
	    ORDER BY Priority, InternalKey;

	RETURN ISNULL(@@ROWCOUNT,0);

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add stored procedure dbo.pds_NSecondaryRegistrySelect')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE TRIGGER [dbo].[TR_NResourceInsertUpdate]
   ON  [dbo].[pds_NResource]
   AFTER INSERT, UPDATE
AS 
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @rows int, @loops int, @ResourceIid int, @RecordRegistryGuid uniqueidentifier;
	DECLARE @Name nvarchar(128), @Nature nvarchar(256), @TagToken char(11);
	
	SELECT @ResourceIid = R.ResourceIidKey, @RecordRegistryGuid = R.RecordRegistryInfosetGuidRef,
		@Name = R.EntityName, @Nature = R.EntityNature		
		FROM pds_NResource AS R INNER 
		JOIN inserted AS i ON (R.ResourceIidKey = i.ResourceIidKey);

	-- Check for non-null empty or blank strings or strings with double spaces
	SET @Name = REPLACE(RTRIM(LTRIM(@Name)),'  ',' ');
	IF  (@Name IS NOT NULL) AND COALESCE(@Name,'') = '' BEGIN
		SET @Name = NULL; -- reset empty string to null
	END;
	SET @Nature = REPLACE(RTRIM(LTRIM(@Nature)),'  ',' ');
	IF  (@Nature IS NOT NULL) AND COALESCE(@Nature,'') = '' BEGIN
		SET @Nature = NULL; -- reset empty string to null
	END;
	
	UPDATE R SET 
		R.EntityName = @Name,
		R.EntityNature = @Nature,
		R.RecordUpdatedOn = getutcdate() 
		FROM pds_NResource AS R WHERE (R.ResourceIidKey = @ResourceIid);	
		
	-- assure that resource record always has a (priority 0) canonical label
		
	SELECT @rows = COUNT(EntityCanonicalLabel) 
		FROM pds_NEntityCanonicalLabelView WHERE (ResourceIidKey = @ResourceIid);	
					
	IF (@rows = 0) BEGIN
		SET @loops = 0; SET @rows  = 1;
		WHILE (@rows > 0) AND (@loops < 99) BEGIN
			EXEC pds_ARandomHexStrVarChar 11, @TagToken output;
			EXEC @rows = pds_NResourceEntityRegTagExists @TagToken, @RecordRegistryGuid, @ResourceIid;	
		END;
		IF (@rows = 0) EXEC pds_NTagAndLabelInsertUpdatePrincipalTag  @ResourceIid, @TagToken;
	END;
	
	-- assure that resource record always has a priority 255 random alias label
	
	SELECT @rows = COUNT(EntityAliasLabel)
		FROM pds_NEntityAliasLabelView WHERE (ResourceIidRef = @ResourceIid) AND (Priority = 255)
					
	IF (@rows = 0) BEGIN
		SET @loops = 0; SET @rows  = 1;
		WHILE (@rows > 0) AND (@loops < 99) BEGIN
			EXEC pds_ARandomHexStrVarChar 11, @TagToken output;
			EXEC @rows = pds_NResourceEntityRegTagExists @TagToken, @RecordRegistryGuid, @ResourceIid;	
		END;
		IF (@rows = 0) EXEC pds_NTagAndLabelInsertAliasTagToken  @ResourceIid, @TagToken, 255;
	END;
 
END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add trigger dbo.pds_NResource.TR_NResourceInsertUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE TRIGGER [dbo].[TR_NRegistryRestrictionOrInsertUpdate]
   ON  [dbo].[pds_NRegistryRestrictionOr]
   AFTER INSERT, UPDATE
AS 
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @RegGuid uniqueidentifier, @RegXml xml;
	
	SELECT @RegGuid = R.InfosetGuidRef
		FROM pds_NRegistryRestrictionView AS R INNER 
		JOIN inserted AS i ON (R.InternalAndKey = i.InternalAndRef);
	
	EXEC pds_NRegistryRestrictionXml @RegGuid, @RegXml output;

	UPDATE S SET 
		S.EntityOtherMetadata = @RegXml,
		S.RecordUpdatedOn = getutcdate() 
		FROM pds_NResource AS S WHERE (S.InfosetGuidKey = @RegGuid);	
 
END


GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add trigger dbo.pds_NRegistryRestrictionOr.TR_NRegistryRestrictionOrInsertUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE TRIGGER [dbo].[TR_NSupportingTagInsertUpdate] 
   ON  [dbo].[pds_NSupportingTag]
   AFTER INSERT, UPDATE
AS 
BEGIN

	SET NOCOUNT ON;
	DECLARE @IntKey int, @SupTag nvarchar(128);

	SELECT @IntKey = T.InternalKey, @SupTag = T.SupportingTag		
		FROM pds_NSupportingTag AS T INNER 
		JOIN inserted AS i ON (T.InternalKey = i.InternalKey);
	
	-- eliminate any leading, trailing, or double spaces
	SET @SupTag = REPLACE(RTRIM(LTRIM(@SupTag)),'  ',' ');
	
	UPDATE T SET
		T.SupportingTag = @SupTag
		FROM pds_NSupportingTag AS T
		WHERE T.InternalKey = @IntKey;

END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add trigger dbo.pds_NSupportingTag.TR_NSupportingTagInsertUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
-- =============================================
-- Author:		<Author,,Name>
-- Create date: <Create Date,,>
-- Description:	<Description,,>
-- =============================================
CREATE TRIGGER [dbo].[TR_NTagAndLabelInsertUpdate]
   ON  dbo.pds_NTagAndLabel
   AFTER INSERT, UPDATE
AS 
BEGIN

	SET NOCOUNT ON;
	
	DECLARE @InsertedKey int, @CanonicalKey int, @ResourceIid int;
	DECLARE @TagToken nvarchar(128),  @LabelUri nvarchar(256);
	DECLARE @EntityLabel nvarchar(384), @RegistryLabel nvarchar(384), @LastChar nchar(1);
	DECLARE @Priority tinyint, @IsTagIndependent bit, @IsCanonical bit;

	DECLARE @UpdateRequired bit, @NumberReset int, @NumberUpdated int;
	SET @UpdateRequired = 0;
	SET @NumberReset = 0;
	SET @NumberUpdated = 0;

	SELECT @InsertedKey = R.InternalKey, @ResourceIid = R.ResourceIidRef, 
		@Priority = R.Priority, @TagToken = R.TagToken,	@LabelUri = R.LabelUri,
		@IsTagIndependent = R.LabelIsTagIndependent,
		@IsCanonical = R.LabelIsCanonical
		FROM pds_NTagAndLabel AS R INNER 
		JOIN inserted AS i ON (R.InternalKey = i.InternalKey);

	-- Check for non-null empty or blank tags and labels
	IF (@TagToken IS NOT NULL) AND COALESCE(LTRIM(RTRIM(@TagToken)),'') = '' BEGIN
		SET @TagToken = NULL; -- reset empty string to null
		SET @UpdateRequired = 1;
	END;
	IF (@LabelUri IS NOT NULL) AND COALESCE(LTRIM(RTRIM(@LabelUri)),'') = '' BEGIN
		SET @LabelUri = NULL; -- reset empty string to null
		SET @UpdateRequired = 1;
	END;
	
	-- Check for any null independent label
	-- (labeluri cannot be tagindependent if labeluri is null)
	IF (@IsTagIndependent = 1) AND (@LabelUri IS NULL) BEGIN
		SET @IsTagIndependent = 0; -- reset true to false
		SET @UpdateRequired = 1;
	END;

	-- Check for any tag/label both non-null combo
	-- (tagtoken/labeluri cannot both be non-null)
	/*  currently enforced in PDS version 0.6 */
	IF (@TagToken IS NOT NULL) AND (@LabelUri IS NOT NULL) BEGIN
		SET @TagToken = NULL; -- prioritize the non-null LabelUri
		SET @UpdateRequired = 1;
	END;

	-- Check for any null canonical tag/label combo
	-- (tagtoken/labeluri cannot be canonical if both are null)
	IF (@IsCanonical = 1) AND (@TagToken IS NULL) AND (@LabelUri IS NULL) BEGIN
		SET @IsCanonical = 0; -- reset true to false
		SET @UpdateRequired = 1;
	END;

	-- set or find the @CanonicalKey
	IF (@IsCanonical = 1) BEGIN
		SET @CanonicalKey = @InsertedKey;
	END ELSE BEGIN
	-- Find key for canonical tag/label by priority (and tie-breaker internalkey) order
		SELECT TOP 1 
			@CanonicalKey = InternalKey
			FROM pds_NTagAndLabel
			WHERE (ResourceIidRef = @ResourceIid)  -- for same Resource
			ORDER BY Priority, InternalKey; 
	END

	-- reset canonical flag and priority if necessary for inserted record
	IF (@CanonicalKey = @InsertedKey) BEGIN
		IF(@IsCanonical = 0) BEGIN
			SET @IsCanonical = 1; -- reset false to true
			SET @UpdateRequired = 1;	
		END;	
		IF (@Priority <> 0) BEGIN
			SET @Priority = 0; -- reset nonzero to zero
			SET @UpdateRequired = 1;
		END;		
	END;	
	
	-- reset canonical flag if necessary for other records
	IF (@IsCanonical = 1) BEGIN
		UPDATE pds_NTagAndLabel SET 
			LabelIsCanonical = 0
			WHERE (ResourceIidRef = @ResourceIid)  -- for same Resource
			AND (InternalKey <> @CanonicalKey)   -- but not the canonical record
			AND (LabelIsCanonical = 1); -- any record for which true must be reset to false
		SET @NumberReset = ISNULL(@@ROWCOUNT,0);
	END 

	-- Perform update to enforce meaningful data integrity on current record
	IF (@UpdateRequired = 1) BEGIN
		UPDATE pds_NTagAndLabel SET 
			Priority = @Priority,
			TagToken = @TagToken,
			LabelUri = @LabelUri,
			LabelIsCanonical = @IsCanonical
			WHERE (InternalKey = @InsertedKey);
		SET @NumberUpdated = ISNULL(@@ROWCOUNT,0);
	END;
	
	-- Retrieve the RegistryLabel	
	SELECT @RegistryLabel = rgyRRtl.EntityLabel
		FROM pds_NResource AS RR INNER 
		JOIN pds_NResource AS rgyRR ON (RR.RecordRegistryInfosetGuidRef = rgyRR.InfosetGuidKey) INNER
		JOIN pds_NTagAndLabel AS entRRtl ON (RR.ResourceIidKey = entRRtl.ResourceIidRef) INNER
		JOIN pds_NTagAndLabel As rgyRRtl ON (rgyRR.ResourceIidKey = rgyRRtl.ResourceIidRef) AND (rgyRRtl.LabelIsCanonical = 1)
		WHERE (entRRtl.ResourceIidRef = @ResourceIid) AND (entRRtl.InternalKey = @InsertedKey);
	
	IF (RIGHT(@RegistryLabel,1) <> '/') 
		SET @RegistryLabel = @RegistryLabel + '/';
		
	-- Always update the EntityLabel
	SELECT @EntityLabel = COALESCE(LabelUri, @RegistryLabel+LOWER(TagToken))
		FROM pds_NTagAndLabel WHERE (InternalKey = @InsertedKey);
		
	IF (@EntityLabel IS NULL) SET @EntityLabel = newid(); -- failsafe to prevent nulls in uniquekey col	

	UPDATE pds_NTagAndLabel SET EntityLabel = @EntityLabel
		WHERE (InternalKey = @InsertedKey);	

	-- TODO: need to error test fully all scenarios
 
END

GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add trigger dbo.pds_NTagAndLabel.TR_NTagAndLabelInsertUpdate')
END
GO
SET QUOTED_IDENTIFIER OFF
GO
SET ANSI_NULLS ON
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingLabel] ADD 
	CONSTRAINT [PK_NSupportingLabelInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NSupportingLabel.PK_NSupportingLabelInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [PK_NTagAndLabelInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NTagAndLabel.PK_NTagAndLabelInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingTag] ADD 
	CONSTRAINT [PK_NSupportingTagInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NSupportingTag.PK_NSupportingTagInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AUserAccountSystem] ADD 
	CONSTRAINT [PK_AUserAccountSystemIidKey] PRIMARY KEY CLUSTERED 
	(
		[SystemIidKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_AUserAccountSystem.PK_AUserAccountSystemIidKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryRegistry] ADD 
	CONSTRAINT [PK_NSecondaryRegistryInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NSecondaryRegistry.PK_NSecondaryRegistryInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryDirectory] ADD 
	CONSTRAINT [PK_NSecondaryDirectoryInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NSecondaryDirectory.PK_NSecondaryDirectoryInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] ADD 
	CONSTRAINT [PK_NAgentIidKey] PRIMARY KEY CLUSTERED 
	(
		[AgentIidKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NAgent.PK_NAgentIidKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AEntityTypeEnum] ADD 
	CONSTRAINT [PK_AEntityTypeEnumCodeKey] PRIMARY KEY CLUSTERED 
	(
		[CodeKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_AEntityTypeEnum.PK_AEntityTypeEnumCodeKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NLocation] ADD 
	CONSTRAINT [PK_NLocationInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NLocation.PK_NLocationInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgentRegistry] ADD 
	CONSTRAINT [PK_NAgentRegistryInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NAgentRegistry.PK_NAgentRegistryInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NCrossReference] ADD 
	CONSTRAINT [PK_NCrossReferenceInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NCrossReference.PK_NCrossReferenceInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [PK_NResourceIidKey] PRIMARY KEY CLUSTERED 
	(
		[ResourceIidKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NResource.PK_NResourceIidKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_AInfosetStatusEnum] ADD 
	CONSTRAINT [PK_AInfosetStatusEnumCodeKey] PRIMARY KEY CLUSTERED 
	(
		[CodeKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_AInfosetStatusEnum.PK_AInfosetStatusEnumCodeKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionAnd] ADD 
	CONSTRAINT [PK_NRegistryRestrictionAndInternalKey] PRIMARY KEY CLUSTERED 
	(
		[InternalAndKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NRegistryRestrictionAnd.PK_NRegistryRestrictionAndInternalKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionOr] ADD 
	CONSTRAINT [PK_NRegistryRestrictionOrInternal] PRIMARY KEY CLUSTERED 
	(
		[InternalOrKey]
	) ON [PRIMARY];
GO

IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add primary key dbo.pds_NRegistryRestrictionOr.PK_NRegistryRestrictionOrInternal')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE UNIQUE INDEX [IX_NTagsAndLabelsResourceIidPriorityInternalKey] ON [dbo].[pds_NTagAndLabel] ([ResourceIidRef],[Priority],[InternalKey]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NTagAndLabel.IX_NTagsAndLabelsResourceIidPriorityInternalKey')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NSupportingLabelResourceIidPriority] ON [dbo].[pds_NSupportingLabel] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NSupportingLabel.IX_NSupportingLabelResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NCrossReferenceResourceIidPriority] ON [dbo].[pds_NCrossReference] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NCrossReference.IX_NCrossReferenceResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NSecondaryRegistryResourceIidPriority] ON [dbo].[pds_NSecondaryRegistry] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NSecondaryRegistry.IX_NSecondaryRegistryResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NLocationResourceIidPriority] ON [dbo].[pds_NLocation] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NLocation.IX_NLocationResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NSupportingTagResourceIidPriority] ON [dbo].[pds_NSupportingTag] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NSupportingTag.IX_NSupportingTagResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NAgentAspnetSystemUser] ON [dbo].[pds_NAgent] ([AspnetSystemIidRef],[AspnetUserGuidRef]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NAgent.IX_NAgentAspnetSystemUser')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NAgentDnnSystemUser] ON [dbo].[pds_NAgent] ([DnnSystemIidRef],[DnnUserIidRef]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NAgent.IX_NAgentDnnSystemUser')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NAgentOtherSystemUser] ON [dbo].[pds_NAgent] ([OtherSystemIidRef],[OtherUserIidRef],[OtherUserGuidRef]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NAgent.IX_NAgentOtherSystemUser')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
CREATE INDEX [IX_NSecondaryDirectoryResourceIidPriority] ON [dbo].[pds_NSecondaryDirectory] ([ResourceIidRef],[Priority]) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add index dbo.pds_NSecondaryDirectory.IX_NSecondaryDirectoryResourceIidPriority')
END
GO

IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionAnd] ADD 
	CONSTRAINT [UK_NRegistryRestrictionAndIndex] UNIQUE NONCLUSTERED 
(
	[ResourceIidRef],[RestrictionIsLabel],[RestrictionAndIndex]
) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add unique constraint dbo.pds_NRegistryRestrictionAnd.UK_NRegistryRestrictionAndIndex')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [UK_NResourceInfosetGuidKey] UNIQUE NONCLUSTERED 
(
	[InfosetGuidKey]
) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add unique constraint dbo.pds_NResource.UK_NResourceInfosetGuidKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [UK_NResourceRecordHandle] UNIQUE NONCLUSTERED 
(
	[RecordHandle]
) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add unique constraint dbo.pds_NResource.UK_NResourceRecordHandle')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [IX_NTagAndLabelEntityLabel] UNIQUE NONCLUSTERED 
(
	[InternalKey]
) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add unique constraint dbo.pds_NTagAndLabel.IX_NTagAndLabelEntityLabel')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionOr] ADD 
	CONSTRAINT [UK_NRegistryRestrictionOrIndex] UNIQUE NONCLUSTERED 
(
	[InternalAndRef],[RestrictionOrIndex]
) ON [PRIMARY];
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add unique constraint dbo.pds_NRegistryRestrictionOr.UK_NRegistryRestrictionOrIndex')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgentRegistry] ADD 
	CONSTRAINT [FK_NAgentRegistry_NAgent_AgentIid] FOREIGN KEY 
	(
		[AgentIidRef]
	)REFERENCES [dbo].[pds_NAgent](
		[AgentIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgentRegistry.FK_NAgentRegistry_NAgent_AgentIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgentRegistry] ADD 
	CONSTRAINT [FK_NAgentRegistry_NResource_InfosetGuid] FOREIGN KEY 
	(
		[RegistryInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgentRegistry.FK_NAgentRegistry_NResource_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NLocation] ADD 
	CONSTRAINT [FK_NLocation_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NLocation.FK_NLocation_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NCrossReference] ADD 
	CONSTRAINT [FK_NCrossReference_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NCrossReference.FK_NCrossReference_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionAnd] ADD 
	CONSTRAINT [FK_NRegistryRestrictionAnd_NResource_InfosetGuid] FOREIGN KEY 
	(
		[InfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NRegistryRestrictionAnd.FK_NRegistryRestrictionAnd_NResource_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionAnd] ADD 
	CONSTRAINT [FK_NRegistryRestrictionAnd_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NRegistryRestrictionAnd.FK_NRegistryRestrictionAnd_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingTag] ADD 
	CONSTRAINT [FK_NSupportingTag_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSupportingTag.FK_NSupportingTag_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSupportingLabel] ADD 
	CONSTRAINT [FK_NSupportingLabel_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSupportingLabel.FK_NSupportingLabel_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] ADD 
	CONSTRAINT [FK_NAgent_NResource_InfosetGuid] FOREIGN KEY 
	(
		[AgentInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgent.FK_NAgent_NResource_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] WITH NOCHECK ADD 
	CONSTRAINT [FK_NAgentAspnetSystemIid_AUserAccountSystemIid] FOREIGN KEY 
	(
		[AspnetSystemIidRef]
	)REFERENCES [dbo].[pds_AUserAccountSystem](
		[SystemIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgent.FK_NAgentAspnetSystemIid_AUserAccountSystemIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] WITH NOCHECK ADD 
	CONSTRAINT [FK_NAgentDnnSystemIid_AUserAccountSystemIid] FOREIGN KEY 
	(
		[DnnSystemIidRef]
	)REFERENCES [dbo].[pds_AUserAccountSystem](
		[SystemIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgent.FK_NAgentDnnSystemIid_AUserAccountSystemIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NAgent] WITH NOCHECK ADD 
	CONSTRAINT [FK_NAgentOtherSystemIid_AUserAccountSystemIid] FOREIGN KEY 
	(
		[OtherSystemIidRef]
	)REFERENCES [dbo].[pds_AUserAccountSystem](
		[SystemIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NAgent.FK_NAgentOtherSystemIid_AUserAccountSystemIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NTagAndLabel] ADD 
	CONSTRAINT [FK_NTagAndLabel_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NTagAndLabel.FK_NTagAndLabel_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_EntityContact_InfosetGuid] FOREIGN KEY 
	(
		[EntityContactInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_EntityContact_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_EntityOther_InfosetGuid] FOREIGN KEY 
	(
		[EntityOtherInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_EntityOther_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_EntityOwner_InfosetGuid] FOREIGN KEY 
	(
		[EntityOwnerInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_EntityOwner_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_RecordDirectory_InfosetGuid] FOREIGN KEY 
	(
		[RecordDirectoryInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_RecordDirectory_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_RecordRegistrant_InfosetGuid] FOREIGN KEY 
	(
		[RecordRegistrantInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_RecordRegistrant_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_RecordRegistrar_InfosetGuid] FOREIGN KEY 
	(
		[RecordRegistrarInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_RecordRegistrar_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResource_RecordRegistry_InfosetGuid] FOREIGN KEY 
	(
		[RecordRegistryInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResource_RecordRegistry_InfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResourceEntityTypeCode_AEntityTypeEnumCodeKey] FOREIGN KEY 
	(
		[EntityTypeCode]
	)REFERENCES [dbo].[pds_AEntityTypeEnum](
		[CodeKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceEntityTypeCode_AEntityTypeEnumCodeKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResourceInfosetDoorsStatusCode_AInfosetStatusEnumCodeKey] FOREIGN KEY 
	(
		[InfosetDoorsStatusCode]
	)REFERENCES [dbo].[pds_AInfosetStatusEnum](
		[CodeKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceInfosetDoorsStatusCode_AInfosetStatusEnumCodeKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] ADD 
	CONSTRAINT [FK_NResourceInfosetPortalStatusCode_AInfosetStatusEnumCodeKey] FOREIGN KEY 
	(
		[InfosetPortalStatusCode]
	)REFERENCES [dbo].[pds_AInfosetStatusEnum](
		[CodeKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceInfosetPortalStatusCode_AInfosetStatusEnumCodeKey')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] WITH NOCHECK ADD 
	CONSTRAINT [FK_NResourceRecordCreatedBy_NAgentIid] FOREIGN KEY 
	(
		[RecordCreatedByAgentIidRef]
	)REFERENCES [dbo].[pds_NAgent](
		[AgentIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceRecordCreatedBy_NAgentIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] WITH NOCHECK ADD 
	CONSTRAINT [FK_NResourceRecordManagedBy_NAgentIid] FOREIGN KEY 
	(
		[RecordManagedByAgentIidRef]
	)REFERENCES [dbo].[pds_NAgent](
		[AgentIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceRecordManagedBy_NAgentIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NResource] WITH NOCHECK ADD 
	CONSTRAINT [FK_NResourceRecordUpdatedBy_NAgentIid] FOREIGN KEY 
	(
		[RecordUpdatedByAgentIidRef]
	)REFERENCES [dbo].[pds_NAgent](
		[AgentIidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NResource.FK_NResourceRecordUpdatedBy_NAgentIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NRegistryRestrictionOr] ADD 
	CONSTRAINT [FK_NRegistryRestrictionOrAndInternal] FOREIGN KEY 
	(
		[InternalAndRef]
	)REFERENCES [dbo].[pds_NRegistryRestrictionAnd](
		[InternalAndKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NRegistryRestrictionOr.FK_NRegistryRestrictionOrAndInternal')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryRegistry] ADD 
	CONSTRAINT [FK_NSecondaryRegistry_NResource_RegistryInfosetGuid] FOREIGN KEY 
	(
		[RegistryInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSecondaryRegistry.FK_NSecondaryRegistry_NResource_RegistryInfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryRegistry] ADD 
	CONSTRAINT [FK_NSecondaryRegistry_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSecondaryRegistry.FK_NSecondaryRegistry_NResource_ResourceIid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryDirectory] ADD 
	CONSTRAINT [FK_NSecondaryDirectory_NResource_DirectoryInfosetGuid] FOREIGN KEY 
	(
		[DirectoryInfosetGuidRef]
	)REFERENCES [dbo].[pds_NResource](
		[InfosetGuidKey]
	)ON UPDATE NO ACTION ON DELETE NO ACTION 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSecondaryDirectory.FK_NSecondaryDirectory_NResource_DirectoryInfosetGuid')
END
GO
IF @@TRANCOUNT=0 BEGIN TRANSACTION
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [dbo].[pds_NSecondaryDirectory] ADD 
	CONSTRAINT [FK_NSecondaryDirectory_NResource_ResourceIid] FOREIGN KEY 
	(
		[ResourceIidRef]
	)REFERENCES [dbo].[pds_NResource](
		[ResourceIidKey]
	)ON UPDATE CASCADE ON DELETE CASCADE 
GO
IF @@ERROR<>0 
Begin
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
	INSERT INTO #ErrorLog (errno,errdescr) values(@@ERROR,'Failed to add foreign key dbo.pds_NSecondaryDirectory.FK_NSecondaryDirectory_NResource_ResourceIid')
END
GO
IF EXISTS (Select * from #ErrorLog)
BEGIN
	IF @@TRANCOUNT>0 ROLLBACK TRANSACTION
END
ELSE
BEGIN
	IF @@TRANCOUNT>0 COMMIT TRANSACTION
END
IF EXISTS (Select * from #ErrorLog)
BEGIN
	Print 'Database creation script failed'
	GOTO QuitWithErrors
END
ELSE
BEGIN
	Print 'Database creation completed successfully'
END



QuitWithErrors:


